import logging

from .zwave_device import QolsysZWaveDevice

LOGGER = logging.getLogger(__name__)


class QolsysThermostat(QolsysZWaveDevice):

    def __init__(self, thermostat_dict: dict, zwave_dict: dict) -> None:

        super().__init__(zwave_dict)

        self._thermostat_id = thermostat_dict.get("_id")
        self._thermostat_version = thermostat_dict.get("version", "")
        self._thermostat_opr = thermostat_dict.get("opr", "")
        self._thermostat_partition_id = thermostat_dict.get("partition_id", "")
        self._thermostat_name = thermostat_dict.get("thermostat_name", "")
        self._thermostat_node_id = thermostat_dict.get("node_id", "")
        self._thermostat_device_temp_unit = thermostat_dict.get("device_temp_unit", "")  # 'F'
        self._thermostat_current_temp = thermostat_dict.get("current_temp", "")  # "78.0"
        self._thermostat_target_cool_temp = thermostat_dict.get("target_cool_temp", "")  # "78"
        self._thermostat_target_heat_temp = thermostat_dict.get("target_heat_temp", "")  # "65"
        self._thermostat_target_temp = thermostat_dict.get("target_temp", "")
        self._thermostat_power_usage = thermostat_dict.get("power_usage", "")
        self._thermostat_mode = thermostat_dict.get("thermostat_mode", "")  # "2"
        self._thermostat_mode_bitmask = thermostat_dict.get("thermostat_mode_bitmask", "")  # "7,24"
        self._thermostat_fan_mode = thermostat_dict.get("fan_mode", "")  # "0"
        self._thermostat_fan_mode_bitmask = thermostat_dict.get("fan_mode_bitmask", "")  # "67"
        self._thermostat_set_point_mode = thermostat_dict.get("set_point_mode", "")
        self._thermostat_set_point_mode_bitmask = thermostat_dict.get("set_point_mode_bitmask", "")  # "-122,1"
        self._thermostat_created_by = thermostat_dict.get("created_by", "")
        self._thermostat_created_date = thermostat_dict.get("created_date", "")
        self._thermostat_updated_by = thermostat_dict.get("updated_by", "")
        self._thermostat_last_updated_date = thermostat_dict.get("last_updated_date", "")
        self._thermostat_mode_updated_time = thermostat_dict.get("thermostat_mode_updated_time", "")
        self._thermostat_fan_mode_updated_time = thermostat_dict.get("fan_mode_updated_time", "")
        self._thermostat_set_point_mode_updated_time = thermostat_dict.get("set_point_mode_updated_time", "")
        self._thermostat_target_cool_temp_updated_time = thermostat_dict.get("target_cool_temp_updated_time", "")
        self._thermostat_target_heat_temp_updated_time = thermostat_dict.get("target_heat_temp_updated_time", "")
        self._thermostat_current_temp_updated_time = thermostat_dict.get("current_temp_updated_time", "")
        self._thermostat_endpoint = thermostat_dict.get("endpoint", "")
        self._thermostat_paired_status = thermostat_dict.get("paired_status", "")
        self._thermostat_configuration_parameter = thermostat_dict.get("configuration_parameter", "")

    @property
    def thermostat_node_id(self) -> str:
        return self._thermostat_node_id

    @property
    def thermostat_name(self) -> str:
        return self._thermostat_name

    @property
    def thermostat_device_temp_unit(self) -> str:
        return self._thermostat_device_temp_unit

    @property
    def thermostat_current_temp(self) -> str:
        return self._thermostat_current_temp

    @property
    def thermostat_target_cool_temp(self) -> str:
        return self._thermostat_target_cool_temp

    @property
    def thermostat_target_heat_temp(self) -> str:
        return self._thermostat_target_heat_temp

    @property
    def thermostat_target_temp(self) -> str:
        return self._thermostat_target_temp

    @property
    def thermostat_mode(self) -> str:
        return self._thermostat_mode

    @property
    def thermostat_fan_mode(self) -> str:
        return self.thermostat_fan_mode

    @property
    def thermostat_set_point_mode(self) -> str:
        return self._thermostat_set_point_mode

    @thermostat_name.setter
    def thermostat_name(self, value: str) -> None:
        if self._thermostat_name != value:
            LOGGER.debug("Thermostat%s (%s) - name: %s", self.thermostat_node_id, self.thermostat_name, value)
            self._thermostat_name = value
            self.notify()

    @thermostat_device_temp_unit.setter
    def thermostat_device_temp_unit(self, value: str) -> None:
        if self._thermostat_device_temp_unit != value:
            LOGGER.debug("Thermostat%s (%s) - device_temp_unit: %s", self.thermostat_node_id, self.thermostat_name, value)
            self._thermostat_device_temp_unit = value
            self.notify()

    @thermostat_current_temp.setter
    def thermostat_current_temp(self, value: str) -> None:
        if self._thermostat_current_temp != value:
            LOGGER.debug("Thermostat%s (%s) - current_temp: %s", self.thermostat_node_id, self.thermostat_name, value)
            self._thermostat_current_temp = value
            self.notify()

    @thermostat_target_cool_temp.setter
    def thermostat_target_cool_temp(self, value: str) -> None:
        if self._thermostat_target_cool_temp != value:
            LOGGER.debug("Thermostat%s (%s) - target_cool_temp: %s", self.thermostat_node_id, self.thermostat_name, value)
            self._thermostat_target_cool_temp = value
            self.notify()

    @thermostat_target_heat_temp.setter
    def thermostat_target_heat_temp(self, value: str) -> None:
        if self._thermostat_target_heat_temp != value:
            LOGGER.debug("Thermostat%s (%s) - target_heat_temp: %s", self.thermostat_node_id, self.thermostat_name, value)
            self._thermostat_target_heat_temp = value
            self.notify()

    @thermostat_target_temp.setter
    def thermostat_target_temp(self, value: str) -> None:
        if self._thermostat_target_temp != value:
            LOGGER.debug("Thermostat%s (%s) - target_temp: %s", self.thermostat_node_id, self.thermostat_name, value)
            self._thermostat_target_temp = value
            self.notify()

    @thermostat_mode.setter
    def thermostat_mode(self, value: str) -> None:
        if self._thermostat_mode != value:
            LOGGER.debug("Thermostat%s (%s) - mode: %s", self.thermostat_node_id, self.thermostat_name, value)
            self._thermostat_mode = value
            self.notify()

    @thermostat_fan_mode.setter
    def thermostat_fan_mode(self, value: str) -> None:
        if self._thermostat_fan_mode != value:
            LOGGER.debug("Thermostat%s (%s) - fan_mode: %s", self.thermostat_node_id, self.thermostat_name, value)
            self._thermostat_fan_mode = value
            self.notify()

    @thermostat_set_point_mode.setter
    def thermostat_set_point_mode(self, value: str) -> None:
        if self._thermostat_set_point_mode != value:
            LOGGER.debug("Thermostat%s (%s) - set_point_mode: %s", self.thermostat_node_id, self.thermostat_name, value)
            self._thermostat_set_point_mode = value
            self.notify()

    def update_thermostat(self, data: dict) -> None:  # noqa: C901, PLR0912, PLR0915
        # Check if we are updating same none_id
        node_id_update = data.get("node_id", "")
        if node_id_update != self.thermostat_node_id_node_id:
            LOGGER.error(
                "Updating Thermostat '%s' (%s) with Thermostat '%s' (different id)",
                self.thermostat_node_id, self.thermostat_name, node_id_update,
            )
            return

        self.start_batch_update()

        if "version" in data:
            self._thermostat_version = data.get("version")
        if "opr" in data:
            self._thermostat_opr = data.get("opr")
        if "partition_id" in data:
            self._thermostat_partition_id = data.get("partition_id")
        if "thermostat_name" in data:
            self.thermostat_name = data.get("thermostat_name")
        if "device_temp_unit" in data:
            self.thermostat_device_temp_unit = data.get("device_temp_unit")
        if "current_temp" in data:
            self.thermostat_current_temp = data.get("current_temp")
        if "target_cool_temp" in data:
            self.thermostat_target_cool_temp = data.get("target_cool_temp")
        if "target_heat_temp" in data:
            self.thermostat_target_heat_temp = data.get("target_heat_temp")
        if "target_temp" in data:
            self.thermostat_target_temp = data.get("target_temp")
        if "power_usage" in data:
            self._thermostat_power_usage = data.get("power_usage")
        if "thermostat_mode" in data:
            self.thermostat_mode = data.get("thermostat_mode")
        if "thermostat_mode_bitmask" in data:
            self._thermostat_mode_bitmask = data.get("thermostat_mode_bitmask")
        if "fan_mode" in data:
            self.thermostat_fan_mode = data.get("fan_mode")
        if "fan_mode_bitmask" in data:
            self._thermostat_fan_mode_bitmask = data.get("fan_mode_bitmask")
        if "set_point_mode" in data:
            self.thermostat_set_point_mode = data.get("set_point_mode")
        if "set_point_mode_bitmask" in data:
            self._thermostat_set_point_mode_bitmask = data.get("set_point_mode_bitmask")
        if "created_by" in data:
            self._thermostat_created_by = data.get("created_by")
        if "updated_by" in data:
            self._thermostat_updated_by = data.get("updated_by")
        if "last_updated_date" in data:
            self._thermostat_last_updated_date = data.get("last_updated_date")
        if "thermostat_mode_updated_time" in data:
            self._thermostat_mode_updated_time = data.get("thermostat_mode_updated_time")
        if "fan_mode_updated_time" in data:
            self._thermostat_fan_mode_updated_time = data.get("fan_mode_updated_time")
        if "set_point_mode_updated_time" in data:
            self._thermostat_set_point_mode_updated_time = data.get("set_point_mode_updated_time")
        if "target_cool_temp_updated_time" in data:
            self._thermostat_target_cool_temp_updated_time = data.get("target_cool_temp_updated_time")
        if "target_heat_temp_updated_time" in data:
            self._thermostat_target_heat_temp_updated_time = data.get("target_heat_temp_updated_time")
        if "current_temp_updated_time" in data:
            self._thermostat_current_temp_updated_time = data.get("current_temp_updated_time")
        if "paired_status" in data:
            self._thermostat_paired_status = data.get("paired_status")
        if "endpoint" in data:
            self._thermostat_endpoint = data.get("endpoint")
        if "configuration_parameter" in data:
            self._thermostat_configuration_parameter = data.get("configuration_parameter")

        self.end_batch_update()

    def to_dict_thermostat(self) -> dict:
        return {
            "_id": self._thermostat_id,
            "version": self._thermostat_version,
            "opr": self._thermostat_opr,
            "partition_id": self._thermostat_partition_id,
            "thermostat_name": self.thermostat_name,
            "device_temp_unit": self.thermostat_device_temp_unit,
            "current_temp": self.thermostat_current_temp,
            "target_cool_temp": self.thermostat_target_cool_temp,
            "target_heat_temp": self.thermostat_target_heat_temp,
            "target_temp": self.thermostat_target_temp,
            "power_usage": self._thermostat_power_usage,
            "thermostat_mode": self.thermostat_mode,
            "thermostat_mode_bitmask": self._thermostat_mode_bitmask,
            "fan_mode": self.thermostat_fan_mode,
            "fan_mode_bitmask": self._thermostat_fan_mode_bitmask,
            "set_point_mode": self.thermostat_set_point_mode,
            "set_point_mode_bitmask": self._thermostat_set_point_mode_bitmask,
            "created_by": self._thermostat_created_by,
            "updated_by": self._thermostat_updated_by,
            "last_updated_date": self._thermostat_last_updated_date,
            "thermostat_mode_updated_time": self._thermostat_mode_updated_time,
            "fan_mode_updated_time": self._thermostat_fan_mode_updated_time,
            "set_point_mode_updated_time": self._thermostat_set_point_mode_updated_time,
            "target_cool_temp_updated_time": self._thermostat_target_cool_temp_updated_time,
            "target_heat_temp_updated_time": self._thermostat_target_heat_temp_updated_time,
            "current_temp_updated_time": self._thermostat_current_temp_updated_time,
            "paired_status": self._thermostat_paired_status,
            "endpoint": self._thermostat_endpoint,
            "configuration_parameter": self._thermostat_configuration_parameter,
        }



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableTcc(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.TccContentProvider/tcc"
        self._table = "tcc"
        self._abort_on_error = False

        self._columns = [
            "tableName",
            "counter",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        # panel sends content://com.qolsys.qolsysprovider.PowerGDeviceContentProvider/powerg_device' twice
        self._cursor.execute(f"INSERT OR IGNORE INTO {self.table} (tableName,counter) VALUES (?,?)", (
            data.get("tableName"),
            data.get("counter", "")))

        self._db.commit()



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableMasterSlave(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.MasterSlaveContentProvider/master_slave"
        self._table = "master_slave"
        self._abort_on_error = False

        self._columns = [
            "_id",
            "version",
            "opr",
            "partition_id",
            "zone_id",
            "ip_address",
            "mac_address",
            "device_type",
            "created_by",
            "created_date",
            "updated_by",
            "last_updated_date",
            "status",
            "device_name",
            "last_updated_iq_remote_checksum",
            "software_version",
            "upgrade_status",
            "name",
            "bssid",
            "ssid",
            "dhcpInfo",
            "topology",
            "reboot_reason",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,version,opr,partition_id,zone_id,ip_address,mac_address,
                             device_type,created_by,created_date,updated_by,last_updated_date,status,device_name,
                             last_updated_iq_remote_checksum,software_version,upgrade_status,name,bssid,ssid,dhcpInfo,topology)
                              VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("version", ""),
            data.get("opr", ""),
            data.get("partition_id", ""),
            data.get("zone_id", ""),
            data.get("ip_address", ""),
            data.get("mac_address", ""),
            data.get("device_type", ""),
            data.get("created_by", ""),
            data.get("created_date", ""),
            data.get("updated_by", ""),
            data.get("last_updated_date", ""),
            data.get("status", ""),
            data.get("device_name", ""),
            data.get("last_updated_iq_remote_checksum", ""),
            data.get("software_version", ""),
            data.get("upgrade_status", ""),
            data.get("name", ""),
            data.get("bssid", ""),
            data.get("ssid", ""),
            data.get("dhcpInfo", ""),
            data.get("topology", "")))

        self._db.commit()



import logging

from .observable import QolsysObservable

LOGGER = logging.getLogger(__name__)


class QolsysZWaveDevice(QolsysObservable):

    def __init__(self, zwave_dict: dict) -> None:
        super().__init__()

        self._id = zwave_dict.get("_id")
        self._node_id = zwave_dict.get("node_id", "")
        self._node_name = zwave_dict.get("node_name", "")
        self._node_type = zwave_dict.get("node_type", "")
        self._node_status = zwave_dict.get("node_status", "")
        self._partition_id = zwave_dict.get("partition_id", "")
        self._node_secure_cmd_cls = zwave_dict.get("node_secure_cmd_cls", "")
        self._node_battery_level = zwave_dict.get("node_battery_level", "")
        self._node_battery_level_value = zwave_dict.get("node_battery_level_value", "")
        self._is_node_listening_node = zwave_dict.get("is_node_listening_node", "")
        self._basic_report_value = zwave_dict.get("basic_report_value", "")
        self._switch_multilevel_report_value = zwave_dict.get("switch_multilevel_report_value", "")
        self._basic_device_type = zwave_dict.get("basic_device_type", "")
        self._generic_device_type = zwave_dict.get("generic_device_type", "")
        self._specific_device_type = zwave_dict.get("specific_device_type", "")
        self._num_secure_command_class = zwave_dict.get("num_secure_command_class", "")
        self._secure_command_class = zwave_dict.get("secure_command_class", "")
        self._manufacture_id = zwave_dict.get("manufacture_id", "")
        self._product_type = zwave_dict.get("product_type", "")
        self._device_protocol = zwave_dict.get("device_protocol", "")
        self._paired_status = zwave_dict.get("paired_status", "")
        self._is_device_sleeping = zwave_dict.get("is_device_sleeping", "")
        self._is_device_hidden = zwave_dict.get("is_device_hidden", "")
        self._last_updated_date = zwave_dict.get("last_updated_date", "")
        self._command_class_list = zwave_dict.get("command_class_list", "")

    @property
    def node_id(self) -> str:
        return self._node_id

    @property
    def paired_status(self) -> str:
        return self._paired_status

    @property
    def node_battery_level(self) -> str:
        return self._node_battery_level

    @property
    def node_battery_level_value(self) -> str:
        return self._node_battery_level_value

    @property
    def node_status(self) -> str:
        return self._node_status

    @property
    def node_name(self) -> str:
        return self._node_name

    @property
    def node_type(self) -> str:
        return self._node_type

    @property
    def partition_id(self) -> str:
        return self._partition_id

    def update_base(self, data: dict) -> None:  # noqa: C901, PLR0912, PLR0915

        # Check if we are updating same node_id
        node_id_update = data.get("node_id", "")
        if node_id_update != self._node_id:
            LOGGER.error(
                "Updating ZWave%s (%s) with ZWave%s (different node_id)", self.node_id, self.node_name, node_id_update
            )
            return

        self.start_batch_update()

        if "paired_status" in data:
            self.paired_status = data.get("paired_status")
        if "node_battery_level" in data:
            self.node_battery_level = data.get("node_battery_level")
        if "node_battery_level_value" in data:
            self.node_battery_level_value = data.get("node_battery_level_value")
        if "node_status" in data:
            self.node_status = data.get("node_status")
        if "node_name" in data:
            self.node_name = data.get("node_name")
        if "node_type" in data:
            self.node_type = data.get("node_type")
        if "partition_id" in data:
            self.partition_id = data.get("partition_id")
        if "node_secure_cmd_cls" in data:
            self._node_secure_cmd_cls = data.get("node_secure_cmd_cls")
        if "is_node_listening_node" in data:
            self._is_node_listening_node = data.get("is_node_listening_node")
        if "basic_report_value" in data:
            self._basic_report_value = data.get("basic_report_value")
        if "switch_multilevel_report_value" in data:
            self._switch_multilevel_report_value = data.get("switch_multilevel_report_value")
        if "basic_device_type" in data:
            self._basic_device_type = data.get("basic_device_type")
        if "generic_device_type" in data:
            self._generic_device_type = data.get("generic_device_type")
        if "specific_device_type" in data:
            self._specific_device_type = data.get("specific_device_type")
        if "num_secure_command_class" in data:
            self._num_secure_command_class = data.get("num_secure_command_class")
        if "secure_command_class" in data:
            self._secure_command_class = data.get("secure_command_class")
        if "manufacture_id" in data:
            self._manufacture_id = data.get("manufacture_id")
        if "product_type" in data:
            self._product_type = data.get("product_type")
        if "device_protocol" in data:
            self._device_protocol = data.get("device_protocol")
        if "paired_status" in data:
            self.paired_status = data.get("paired_status")
        if "is_device_sleeping" in data:
            self._is_device_sleeping = data.get("is_device_sleeping")
        if "is_device_hidden" in data:
            self._is_device_hidden = data.get("is_device_hidden")
        if "last_updated_date" in data:
            self._last_updated_date = data.get("last_updated_date")
        if "command_class_list" in data:
            self._last_updated_date = data.get("command_class_list")

        self.end_batch_update()

    @node_id.setter
    def node_id(self, value: str) -> str:
        self._node_id = value

    @paired_status.setter
    def paired_status(self, value: str) -> str:
        if self._paired_status != value:
            LOGGER.debug("ZWave%s (%s) - paired_status: %s", self.node_id, self.node_name, value)
            self._paired_status = value
            self.notify()

    @node_battery_level.setter
    def node_battery_level(self, value: str) -> None:
        if self._node_battery_level != value:
            LOGGER.debug("ZWave%s (%s) - node_battery_level: %s", self.node_id, self.node_name, value)
            self._node_battery_level = value
            self.notify()

    @node_battery_level_value.setter
    def node_battery_level_value(self, value: str) -> str:
        if self._node_battery_level_value != value:
            LOGGER.debug("ZWave%s (%s) - node_battery_level_value: %s", self.node_id, self.node_name, value)
            self._node_battery_level_value = value
            self.notify()

    @node_status.setter
    def node_status(self, value: str) -> str:
        if self._node_status != value:
            LOGGER.debug("ZWave%s (%s) - node_status: %s", self.node_id, self.node_name, value)
            self._node_status = value
            self.notify()

    @node_name.setter
    def node_name(self, value: str) -> str:
        if self._node_name != value:
            LOGGER.debug("ZWave%s (%s) - node_name: %s", self.node_id, self.node_name, value)
            self._node_name = value
            self.notify()

    @node_type.setter
    def node_type(self, value: str) -> str:
        if self._node_type != value:
            LOGGER.debug("ZWave%s (%s) - node_type: %s", self.node_id, self.node_name, value)
            self._node_type = value
            self.notify()

    @partition_id.setter
    def partition_id(self, value: str) -> str:
        if self._partition_id != value:
            LOGGER.debug("ZWave%s (%s) - partition_id: %s", self._node_id, self._node_name, value)
            self._partition_id = value
            self.notify()

    def to_dict_base(self) -> dict:
        return {
            "_id": self._id,
            "node_id": self.node_id,
            "node_name": self.node_name,
            "node_type": self.node_type,
            "node_status": self.node_status,
            "partition_id": self._partition_id,
            "node_battery_level": self.node_battery_level,
            "node_battery_level_value": self.node_battery_level_value,
            "paired_status": self.paired_status,
            "node_secure_cmd_cls": self._node_secure_cmd_cls,
            "is_node_listening_node": self._is_node_listening_node,
            "basic_report_value": self._basic_report_value,
            "switch_multilevel_report_value": self._switch_multilevel_report_value,
            "basic_device_type": self._basic_device_type,
            "generic_device_type": self._generic_device_type,
            "specific_device_type": self._specific_device_type,
            "num_secure_command_class": self._num_secure_command_class,
            "secure_command_class": self._secure_command_class,
            "manufacture_id": self._manufacture_id,
            "product_type": self._product_type,
            "device_protocol": self._device_protocol,
            "is_device_sleeping": self._is_device_sleeping,
            "is_device_hidden": self._is_device_hidden,
            "last_updated_date": self._last_updated_date,
            "command_class_list": self._command_class_list,
        }



import logging
from collections.abc import Callable

LOGGER = logging.getLogger(__name__)


class QolsysObservable:
    def __init__(self) -> None:
        self._observers: list[Callable[[], None]] = []

        self._batch_update_active = False
        self._batch_update_change_detected = False

    def register(self, observer: Callable[[], None]) -> None:
        self._observers.append(observer)

    def unregister(self, observer: Callable[[], None]) -> None:
        self._observers.remove(observer)

    def notify(self, **payload) -> None:  # noqa: ANN003
        if self._batch_update_active:
            self._batch_update_change_detected = True
        else:
            for observer in self._observers:
                observer(self, **payload)

    def start_batch_update(self) -> None:
        self._batch_update_change_detected = False
        self._batch_update_active = True

    def end_batch_update(self) -> None:
        self._batch_update_active = False
        if self._batch_update_change_detected:
            self.notify()



#!/usr/bin/env python3
import logging

from .panel import QolsysPanel
from .plugin_c4 import QolsysPluginC4
from .plugin_remote import QolsysPluginRemote
from .settings import QolsysSettings
from .state import QolsysState

LOGGER = logging.getLogger(__name__)


class QolsysController:

    def __init__(self) -> None:

        # QolsysController Information
        self.plugin = None
        self._state = QolsysState()
        self._settings = QolsysSettings()
        self._panel = QolsysPanel(settings=self.settings, state=self.state)

    @property
    def state(self) -> QolsysState:
        return self._state

    @property
    def panel(self) -> QolsysPanel:
        return self._panel

    @property
    def settings(self) -> QolsysSettings:
        return self._settings

    def select_plugin(self, plugin: str) -> None:

        match plugin:

            case "c4":
                LOGGER.debug("C4 Plugin Selected")
                self.plugin = QolsysPluginC4(self.state, self.panel, self.settings)
                return

            case "remote":
                LOGGER.debug("Remote Plugin Selected")
                self.plugin = QolsysPluginRemote(self.state, self.panel, self.settings)
                return

            case _:
                LOGGER.debug("Unknow Plugin Selected")



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableZwaveHistory(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.ZDeviceHistoryContentProvider/zwave_history"
        self._table = "zwave_history"
        self._abort_on_error = False

        self._columns = [
            "_id",
            "version",
            "opr",
            "partition_id",
            "node_id",
            "device_name",
            "source",
            "event",
            "request",
            "response",
            "created_date",
            "updated_date",
            "last_updated_by",
            "field_type",
            "ack",
            "protocol",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,version,opr,partition_id,node_id,device_name,source,event,
                             request,response,created_date,updated_date,last_updated_by,field_type,ack,protocol)
                              VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("version", ""),
            data.get("opr", ""),
            data.get("partition_id", ""),
            data.get("node_id", ""),
            data.get("device_name", ""),
            data.get("source", ""),
            data.get("event", ""),
            data.get("request", ""),
            data.get("response", ""),
            data.get("created_date", ""),
            data.get("updated_date", ""),
            data.get("last_updated_by", ""),
            data.get("field_type", ""),
            data.get("ack", ""),
            data.get("protocol", "")))

        self._db.commit()



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableAlarmedSensor(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.AlarmedSensorProvider/alarmedsensor"
        self._table = "alarmedsensor"
        self._abort_on_error = True

        self._columns = [
            "_id",
            "partition_id",
            "silenced",
            "zone_id",
            "sgroup",
            "action",
            "timed_out",
            "type",
            "priority",
            "aseb_type",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,partition_id,silenced,zone_id,sgroup,action,timed_out,type,
                             priority,aseb_type) VALUES (?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("partition_id", ""),
            data.get("silenced", ""),
            data.get("zone_id", ""),
            data.get("sgroup", ""),
            data.get("action", ""),
            data.get("timed_out", ""),
            data.get("type", ""),
            data.get("priority", ""),
            data.get("aseb_type", "")))

        self._db.commit()



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableWeather(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.ForecastWeatherContentProvider/weather"
        self._table = "weather"
        self._abort_on_error = False

        self._columns = [
            "_id",
            "version",
            "opr",
            "partition_id",
            "high_temp",
            "low_temp",
            "day_of_week",
            "condition",
            "icon",
            "precipitation",
            "current_weather_date",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,version,opr,partition_id,high_temp,low_temp,day_of_week,
                             condition,icon,precipitation,current_weather_date) VALUES (?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("version", ""),
            data.get("opr", ""),
            data.get("partition_id", ""),
            data.get("high_temp", ""),
            data.get("low_temp", ""),
            data.get("day_of_week", ""),
            data.get("condition", ""),
            data.get("icon", ""),
            data.get("precipitation", ""),
            data.get("current_weather_date", "")))

        self._db.commit()



import logging

from .observable import QolsysObservable
from .partition import QolsysPartition
from .zone import QolsysZone
from .zwave_device import QolsysZWaveDevice
from .zwave_dimmer import QolsysDimmer
from .zwave_generic import QolsysGeneric
from .zwave_lock import QolsysLock
from .zwave_thermostat import QolsysThermostat

LOGGER = logging.getLogger(__name__)


class QolsysState(QolsysObservable):

    def __init__(self) -> None:
        super().__init__()

        self._partitions = []
        self._zones = []
        self._zwave_devices = []

        self._state_partition_observer = QolsysObservable()
        self._state_zone_observer = QolsysObservable()
        self._state_zwave_observer = QolsysObservable()

    @property
    def partitions(self) -> list[QolsysPartition]:
        return self._partitions

    @property
    def zwave_devices(self) -> list[QolsysZWaveDevice]:
        return self._zwave_devices

    @property
    def zones(self) -> list[QolsysZone]:
        return self._zones

    @property
    def zwave_dimmers(self) -> list[QolsysDimmer]:
        dimmers = []
        for device in self.zwave_devices:
            if isinstance(device, QolsysDimmer):
                dimmers.append(device)

        return dimmers

    @property
    def zwave_locks(self) -> list[QolsysLock]:
        locks = []
        for device in self.zwave_devices:
            if isinstance(device, QolsysLock):
                locks.append(device)

        return locks

    @property
    def zwave_thermostats(self) -> list[QolsysThermostat]:
        thermostats = []
        for device in self.zwave_devices:
            if isinstance(device, QolsysThermostat):
                thermostats.append(device)

        return thermostats

    @property
    def state_partition_observer(self) -> QolsysObservable:
        return self._state_partition_observer

    @property
    def state_zone_observer(self) -> QolsysObservable:
        return self._state_zone_observer

    @property
    def state_zwave_observer(self) -> QolsysObservable:
        return self._state_zwave_observer

    def partition(self, partition_id: str) -> QolsysPartition:
        for partition in self.partitions:
            if partition.id == partition_id:
                return partition

        return None

    def partition_add(self, new_partition: QolsysPartition) -> None:
        for partition in self.partitions:
            if new_partition.id == partition.id:
                LOGGER.debug("Adding Partition to State, Partition%s (%s) - Allready in Partitions List",
                             new_partition.id, partition.name)
                return

        self.partitions.append(new_partition)
        self.state_partition_observer.notify()

    def partition_delete(self, partition_id: str) -> None:
        partition = self.partition(partition_id)

        if partition is None:
            LOGGER.debug("Deleting Partition from State, Partition%s not found", partition_id)
            return

        self.partitions.remove(partition)
        self.state_partition_observer.notify()

    def zone(self, zone_id: str) -> QolsysZone:
        for zone in self.zones:
            if zone.zone_id == zone_id:
                return zone

        return None

    def zone_add(self, new_zone: QolsysZone) -> None:
        for zone in self.zones:
            if new_zone.zone_id == zone.zone_id:
                LOGGER.debug("Adding Zone to State, zone%s (%s) - Allready in Zone List", new_zone.zone_id, self.sensorname)
                return

        self.zones.append(new_zone)
        self.state_zone_observer.notify()

    def zone_delete(self, zone_id: str) -> None:
        zone = self.zone(zone_id)

        if zone is None:
            LOGGER.debug("Deleting Zone from State, Zone%s not found", zone_id)
            return

        self.zones.remove(zone)
        self.state_zone_observer.notify()

    def zwave_device(self, node_id: str) -> QolsysZWaveDevice:
        for zwave_device in self.zwave_devices:
            if zwave_device.node_id == node_id:
                return zwave_device

        return None

    def zwave_add(self, new_zwave: QolsysZWaveDevice) -> None:
        for zwave_device in self.zwave_devices:
            if new_zwave.node_id == zwave_device.node_id:
                LOGGER.debug("Adding ZWave to State, ZWave%s (%s) - Allready in ZWave List",
                             new_zwave.node_id, zwave_device.node_name)
                return

        self.zwave_devices.append(new_zwave)
        self.state_zwave_observer.notify()

    def zwave_delete(self, node_id: str) -> None:
        zwave = self.zwave_device(node_id)

        if zwave is None:
            LOGGER.debug("Deleting ZWave from State, ZWave%s not found", node_id)
            return

        self.zwave_devices.remove(zwave)
        self.state_zwave_observer.notify()

    def sync_zwave_devices_data(self, db_zwaves: list[QolsysZWaveDevice]) -> None:  # noqa: C901, PLR0912

        db_zwave_list = []
        for db_zwave in db_zwaves:
            db_zwave_list.append(db_zwave.node_id)

        state_zwave_list = []
        for state_zwave in self.zwave_devices:
            state_zwave_list.append(state_zwave.node_id)

        # Update existing ZWave devices
        for state_zwave in self.zwave_devices:
            if state_zwave.node_id in db_zwave_list:
                for db_zwave in db_zwaves:
                    if state_zwave.node_id == db_zwave.node_id:
                        LOGGER.debug("sync_data - update ZWave%s", state_zwave.node_id)

                        # Update Dimmer
                        if isinstance(state_zwave, QolsysDimmer) and isinstance(db_zwave, QolsysDimmer):
                            state_zwave.update_base(db_zwave.to_dict_base())
                            state_zwave.update_dimmer(db_zwave.to_dict_dimmer())
                            break

                        # Update Thermostat
                        if isinstance(state_zwave, QolsysThermostat) and isinstance(db_zwave, QolsysThermostat):
                            state_zwave.update_base(db_zwave.to_dict_base())
                            state_zwave.update_thermostat(db_zwave.to_dict_base())
                            break

                        # Update Lock
                        if isinstance(state_zwave, QolsysLock) and isinstance(db_zwave, QolsysLock):
                            state_zwave.update_base(db_zwave.to_dict_base())
                            state_zwave.update_lock(db_zwave.to_dict_base())
                            break

                        # Generic Z-Wave Device
                        if isinstance(state_zwave, QolsysGeneric) and isinstance(db_zwave, QolsysGeneric):
                            state_zwave.update_base(db_zwave.to_dict_base())
                            break

                        # zwave node_id has changed of node_type, delete and add again
                        # self.zwave_delete(int(state_zwave.node_id))  # noqa: ERA001
                        # self.zwave_add(db_zwave)  # noqa: ERA001

        # Add new zwave device
        for db_zwave in db_zwaves:
            if db_zwave.node_id not in state_zwave_list:
                LOGGER.debug("sync_data - add ZWave%s", db_zwave.node_id)
                self.zwave_add(db_zwave)

        # Delete zwave device
        for state_zwave in self.zwave_devices:
            if state_zwave.node_id not in db_zwave_list:
                LOGGER.debug("sync_data - delete ZWave%s", state_zwave.none_id)
                self.zwave_delete(int(state_zwave.node_id))

    def sync_zones_data(self, db_zones: list[QolsysZone]) -> None:  # noqa: C901
        db_zone_list = []
        for db_zone in db_zones:
            db_zone_list.append(db_zone.zone_id)

        state_zone_list = []
        for state_zone in self.zones:
            state_zone_list.append(state_zone.zone_id)

        # Update existing zones
        for state_zone in self.zones:
            if state_zone.zone_id in db_zone_list:
                for db_zone in db_zones:
                    if state_zone.zone_id == db_zone.zone_id:
                        LOGGER.debug("sync_data - update Zone%s", state_zone.zone_id)
                        state_zone.update(db_zone.to_dict())

        # Delete zones
        for state_zone in self.zones:
            if state_zone.zone_id not in db_zone_list:
                LOGGER.debug("sync_data - delete Zone%s", state_zone.zone_id)
                self.zone_delete(int(state_zone.zone_id))

        # Add new zone
        for db_zone in db_zones:
            if db_zone.zone_id not in state_zone_list:
                LOGGER.debug("sync_data - add Zone%s", db_zone.zone_id)
                self.zone_add(db_zone)

    def sync_partitions_data(self, db_partitions: list[QolsysPartition]) -> None:  # noqa: C901
        db_partition_list = []
        for db_partition in db_partitions:
            db_partition_list.append(db_partition.id)

        state_partition_list = []
        for state_partition in self.partitions:
            state_partition_list.append(state_partition.id)

        # Update existing partitions
        for state_partition in self.partitions:
            if state_partition.id in db_partition_list:
                for db_partition in db_partitions:
                    if state_partition.id == db_partition.id:
                        LOGGER.debug("sync_data - update Partition%s", state_partition.id)
                        state_partition.update_partition(db_partition.to_dict_partition())
                        state_partition.update_settings(db_partition.to_dict_settings())
                        state_partition.alarm_type_array = db_partition.alarm_type_array
                        state_partition.alarm_state = db_partition.alarm_state

        # Delete partitions
        for state_partition in self.partitions:
            if state_partition.id not in db_partition_list:
                LOGGER.debug("sync_data - delete Partition%s", state_partition.id)
                self.partition_delete(state_partition.id)

        # Add new partition
        for db_partition in db_partitions:
            if db_partition.id not in state_partition_list:
                LOGGER.debug("sync_data - Add Partition%s", db_partition.id)
                self.partition_add(db_partition)

    def dump(self) -> None:  # noqa: PLR0915
        LOGGER.debug("*** Information ***")

        for partition in self.partitions:
            pid = partition.id
            name = partition.name
            LOGGER.debug("Partition%s (%s) - system_status: %s", pid, name, partition.system_status)
            LOGGER.debug("Partition%s (%s) - system_status_changed_time: %s", pid, name, partition.system_status_changed_time)
            LOGGER.debug("Partition%s (%s) - alarm_state: %s", pid, name, partition.alarm_state)

            if partition.alarm_type_array == []:
                LOGGER.debug("Partition%s (%s) - alarm_type: %s", pid, name, "None")
            else:
                for alarm_type in partition.alarm_type_array:
                    LOGGER.debug("Partition%s (%s) - alarm_type: %s", pid, name, alarm_type)

            LOGGER.debug("Partition%s (%s) - exit_sounds: %s", pid, name, partition.exit_sounds)
            LOGGER.debug("Partition%s (%s) - entry_delays: %s", pid, name, partition.entry_delays)

        for zone in self.zones:
            zid = zone.zone_id
            name = zone.sensorname
            LOGGER.debug("Zone%s (%s) - status: %s", zid, name, zone.sensorstatus)
            LOGGER.debug("Zone%s (%s) - battery_status: %s", zid, name, zone.battery_status)
            LOGGER.debug("Zone%s (%s) - latestdBm: %s", zid, name, zone.latestdBm)
            LOGGER.debug("Zone%s (%s) - averagedBm: %s", zid, name, zone.averagedBm)

        for zwave in self.zwave_devices:
            if isinstance(zwave, QolsysDimmer):
                nid = zwave.node_id
                name = zwave.dimmer_name
                LOGGER.debug("Dimmer%s (%s) - status: %s", nid, name, zwave.dimmer_status)
                LOGGER.debug("Dimmer%s (%s) - level: %s", nid, name, zwave.dimmer_level)
                LOGGER.debug("Dimmer%s (%s) - paired_status: %s", nid, name, zwave.paired_status)
                LOGGER.debug("Dimmer%s (%s) - node_status: %s", nid, name, zwave.node_status)
                LOGGER.debug("Dimmer%s (%s) - battery_level: %s", nid, name, zwave.node_battery_level)
                LOGGER.debug("Dimmer%s (%s) - battery_level_value: %s", nid, name, zwave.node_battery_level_value)
                continue

            if isinstance(zwave, QolsysThermostat):
                zid = zwave.thermostat_node_id
                name = zwave.thermostat_name
                LOGGER.debug("Thermostat%s (%s) - current_temp: %s", zid, name, zwave.thermostat_current_temp)
                LOGGER.debug("Thermostat%s (%s) - mode: %s", zid, name, zwave.thermostat_mode)
                LOGGER.debug("Thermostat%s (%s) - fan_mode: %s", zid, name, zwave.thermostat_fan_mode)
                LOGGER.debug("Thermostat%s (%s) - target_temp: %s", zid, name, zwave.thermostat_target_temp)
                LOGGER.debug("Thermostat%s (%s) - target_cool_temp: %s", zid, name, zwave.thermostat_target_cool_temp)
                LOGGER.debug("Thermostat%s (%s) - target_heat_temp: %s", zid, name, zwave.thermostat_target_heat_temp)
                LOGGER.debug("Thermostat%s (%s) - set_point_mode: %s", zid, name, zwave.thermostat_set_point_mode)
                continue

            if isinstance(zwave, QolsysLock):
                zid = zwave.lock_node_id
                name = zwave.lock_name
                LOGGER.debug("Lock%s (%s) - current_temp: %s", zid, name, zwave.lock_status)
                continue

            if isinstance(zwave, QolsysGeneric):
                zid = zwave.node_id
                name = zwave.node_name
                LOGGER.debug("Generic%s (%s) - node_type: %s", zid, name, zwave.node_type)
                LOGGER.debug("Generic%s (%s) - status: %s", zid, name, zwave.node_status)
                LOGGER.debug("Generic%s (%s) - battery_level: %s", zid, name, zwave.node_battery_level)
                LOGGER.debug("Generic%s (%s) - battery_level_vale: %s", zid, name, zwave.node_battery_level_value)
                continue



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableSensor(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.SensorContentProvider/sensor"
        self._table = "sensor"
        self._abort_on_error = True

        self._columns = [
            "_id",
            "version",
            "opr",
            "partition_id",
            "ac_status",
            "sensorid",
            "sensortype",
            "sensorname",
            "sensorgroup",
            "chimetype",
            "sensorstatus",
            "time",
            "sensorstate",
            "sensortts",
            "zoneid",
            "frame_id",
            "zone_alarm_type",
            "zone_equipment_code",
            "zone_physical_type",
            "zone_type",
            "zone_rf_sensor",
            "zone_supervised",
            "zone_two_way_voice_enabled",
            "zone_reporting_enabled",
            "battery_status",
            "created_date",
            "created_by",
            "updated_date",
            "updated_by",
            "frame_count",
            "frame_type",
            "current_capability",
            "shortID",
            "diag_24hr",
            "allowdisarming",
            "device_capability",
            "sub_type",
            "signal_source",
            "powerg_manufacture_id",
            "parent_node",
            "latestdBm",
            "averagedBm",
            "serial_number",
            "extras",
            "allowspeaker",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,version,opr,partition_id,sensorid,sensortype,sensorname,
                             sensorgroup,chimetype,sensorstatus,time,sensorstate,sensortts,zoneid,frame_id,zone_alarm_type,
                             zone_equipment_code,zone_physical_type,zone_type,zone_rf_sensor,zone_supervised,
                             zone_two_way_voice_enabled, zone_reporting_enabled, battery_status,created_date,created_by,
                             updated_date,updated_by,frame_count,frame_type,current_capability,shortID,diag_24hr,
                             allowdisarming,device_capability,sub_type, signal_source, powerg_manufacture_id,parent_node,
                             latestdBm,averagedBm,serial_number,extras,ac_status,allowspeaker)
                              VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,
                             ?,?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("version", ""),
            data.get("opr", ""),
            data.get("partition_id", ""),
            data.get("sensorid", ""),
            data.get("sensortype", ""),
            data.get("sensorname", ""),
            data.get("sensorgroup", ""),
            data.get("chimetype", ""),
            data.get("sensorstatus", ""),
            data.get("time", ""),
            data.get("sensorstate", ""),
            data.get("sensortts", ""),
            data.get("zoneid", ""),
            data.get("frame_id", ""),
            data.get("zone_alarm_type", ""),
            data.get("zone_equipment_code", ""),
            data.get("zone_physical_type", ""),
            data.get("zone_type", ""),
            data.get("zone_rf_sensor", ""),
            data.get("zone_supervised", ""),
            data.get("zone_two_way_voice_enabled", ""),
            data.get("zone_reporting_enabled", ""),
            data.get("battery_status", ""),
            data.get("created_date", ""),
            data.get("created_by", ""),
            data.get("updated_date", ""),
            data.get("updated_by", ""),
            data.get("frame_count", ""),
            data.get("frame_type", ""),
            data.get("current_capability", ""),
            data.get("shortID", ""),
            data.get("diag_24hr", ""),
            data.get("allowdisarming", ""),
            data.get("device_capability", ""),
            data.get("sub_type", ""),
            data.get("signal_source", ""),
            data.get("powerg_manufacture_id", ""),
            data.get("parent_node", ""),
            data.get("latestdBm", ""),
            data.get("averagedBm", ""),
            data.get("serial_number", ""),
            data.get("extras", ""),
            data.get("ac_status", ""),
            data.get("allowspeaker", "")))

        self._db.commit()



import logging

from .enum import ZoneSensorGroup, ZoneSensorType, ZoneStatus
from .observable import QolsysObservable

LOGGER = logging.getLogger(__name__)


class QolsysZone(QolsysObservable):

    def __init__(self, data: dict) -> None:
        super().__init__()

        self._zone_id = data.get("zoneid", "")
        self._sensorname = data.get("sensorname", "")
        self._sensorstatus: ZoneStatus = ZoneStatus(data.get("sensorstatus", ""))
        self._sensortype = ZoneSensorType(data.get("sensortype", ""))
        self._sensorgroup = data.get("sensorgroup", "")
        self._battery_status = data.get("battery_status", "")
        self._averagedBm = data.get("averagedBm", "")
        self._latestdBm = data.get("latestdBm", "")
        self._ac_status = data.get("ac_status", "")

        self._id = data.get("_id", "")
        self._zone_type = data.get("zone_type", "")
        self._sensor_id = data.get("sensorid", "")
        self._sensorstate = data.get("sensorstate", "")
        self._zone_physical_type = data.get("zone_physical_type", "")
        self._zone_alarm_type = data.get("zone_alarm_type", "")
        self._partition_id = data.get("partition_id", "")
        self._sensortts = data.get("sensortts", "")
        self._current_capability = data.get("current_capability", "")
        self._zone_rf_sensor = data.get("zone_rf_sensor", "")
        self._zone_supervised = data.get("zone_supervised", "")
        self._zone_reporting_enabled = data.get("zone_reporting_enabled", "")
        self._zone_two_way_voice_enabled = data.get("zone_two_way_voice_enabled", "")
        self._signal_source = data.get("signal_source", "")
        self._serial_number = data.get("serial_number", "")
        self._chimetype = data.get("chimetype", "")
        self._frame_count = data.get("frame_count", "")
        self._frame_type = data.get("frame_type", "")
        self._frame_id = data.get("frame_id", "")
        self._allowdisarming = data.get("allowdisarming", "")
        self._time = data.get("time", "")
        self._version = data.get("version", "")
        self._opr = data.get("opr", "")
        self._zone_equipement_code = data.get("zone_equipment_code", "")
        self._created_date = data.get("created_date", "")
        self._created_by = data.get("created_by", "")
        self._updated_by = data.get("updated_by", "")
        self._updated_date = data.get("updated_date", "")
        self._shortID = data.get("shortID", "")
        self._diag_24hr = data.get("diag_24hr", "")
        self._device_capability = data.get("device_capability", "")
        self._sub_type = data.get("sub_type", "")
        self._powerg_manufacture_id = data.get("powerg_manufacture_id", "")
        self._parent_node = data.get("parent_node", "")
        self._extras = data.get("extras", "")

    def update(self, data: dict) -> None:  # noqa: C901, PLR0912, PLR0915

        zone_id_update = data.get("zoneid", "")
        if zone_id_update != self._zone_id:
            LOGGER.error("Updating Zone%s (%s) with Zone%s (different id)", self._zone_id, self.sensorname, zone_id_update)
            return

        self.start_batch_update()

        # Update sensor_name
        if "sensorname" in data:
            self.sensorname = data.get("sensorname")

        # Update sensorsatus
        if "sensorstatus" in data:
            self.sensorstatus = ZoneStatus(data.get("sensorstatus"))

        # Update battery_status
        if "battery_status" in data:
            self.battery_status = data.get("battery_status")

        # Update time
        if "time" in data:
            self.time = data.get("time")

        # Update partition_id
        if "partition_id" in data:
            self._partition_id = data.get("partition_id")

        # Update lastestdBm
        if "lastestdBm" in data:
            self.latestdBm = data.get("latestdBm")

        # Update averagedBm
        if "averagedBm" in data:
            self.averagedBm = data.get("averagedBm")

        if "sensorgroup" in data:
            self.sensorgroup = data.get("sensorgroup")

        if "sensorstate" in data:
            self._sensorstate = data.get("sensorstate")

        if "sensortype" in data:
            self.sensortype = ZoneSensorType(data.get("sensortype"))

        if "zone_type" in data:
            self._zone_type = data.get("zone_type")

        if "zone_physical_type" in data:
            self._zone_physical_type = data.get("zone_physical_type")

        if "zone_alarm_type" in data:
            self._zone_alarm_type = data.get("zone_alarm_type")

        if "sensorttss" in data:
            self._sensortts = data.get("sensortts")

        if "current_capability" in data:
            self._current_capability = data.get("current_capability")

        if "zone_rf_sensor" in data:
            self._zone_rf_sensor = data.get("zone_rf_sensor")

        if "zone_supervised" in data:
            self._zone_supervised = data.get("zone_supervised")

        if "zone_reporting_enabled" in data:
            self._zone_reporting_enabled = data.get("zone_reporting_enabled")

        if "zone_two_way_voice_enabled" in data:
            self._zone_two_way_voice_enabled = data.get("zone_two_way_voice_enabled")

        if "signal_source" in data:
            self._signal_source = data.get("signal_source")

        if "serial_number" in data:
            self._serial_number = data.get("serial_number")

        if "chimetype" in data:
            self._chimetype = data.get("chimetype")

        if "frame_count" in data:
            self._frame_count = data.get("frame_count")

        if "frame_type" in data:
            self._frame_type = data.get("frame_type")

        if "allowdisarming" in data:
            self._allowdisarming = data.get("allowdisarming")

        self.end_batch_update()

    @property
    def id(self) -> str:
        return self._id

    @property
    def sensorname(self) -> str:
        return self._sensorname

    @property
    def sensorgroup(self) -> str:
        return self._sensorgroup

    @property
    def sensorstatus(self) -> ZoneStatus:
        return self._sensorstatus

    @property
    def battery_status(self) -> str:
        return self._battery_status

    @property
    def sensorstate(self) -> str:
        return self._sensorstate

    @property
    def sensortype(self) -> ZoneSensorType:
        return self._sensortype

    @property
    def zone_id(self) -> str:
        return self._zone_id

    @property
    def zone_type(self) -> str:
        return self._zone_type

    @property
    def zone_physical_type(self) -> str:
        return self._zone_physical_type

    @property
    def zone_alarm_type(self) -> str:
        return self._zone_alarm_type

    @property
    def partition_id(self) -> str:
        return self._partition_id

    @property
    def time(self) -> str:
        return self._time

    @property
    def latestdBm(self) -> str:
        return self._latestdBm

    @property
    def averagedBm(self) -> str:
        return self._averagedBm

    @averagedBm.setter
    def averagedBm(self, value: str) -> None:
        if self._averagedBm != value:
            self._averagedBm = value
            self.notify()

    @latestdBm.setter
    def latestdBm(self, value: str) -> None:
        if self._latestdBm != value:
            self.latestdBm = value
            self.notify()

    @sensorstatus.setter
    def sensorstatus(self, new_value: ZoneStatus) -> None:
        if self._sensorstatus != new_value:
            LOGGER.debug("Zone%s (%s) - sensorstatus: %s", self._zone_id, self.sensorname, new_value)
            self._sensorstatus = new_value
            self.notify()

    @battery_status.setter
    def battery_status(self, value: str) -> None:
        if self._battery_status != value:
            LOGGER.debug("Zone%s (%s) - battery_status: %s", self.zone_id, self.sensorname, value)
            self._battery_status = value
            self.notify()

    @sensorname.setter
    def sensorname(self, value: str) -> None:
        if self.sensorname != value:
            self._sensorname = value
            self.notify()

    @time.setter
    def time(self, value: str) -> None:
        if self._time != value:
            LOGGER.debug("Zone%s (%s) - time: %s", self.zone_id, self.sensorname, value)
            self._time = value
            self.notify()

    @sensortype.setter
    def sensortype(self, value: ZoneSensorType) -> None:
        if self._sensortype != value:
            self._sensortype = value
            self.notify()

    @sensorgroup.setter
    def sensorgroup(self, new_value: str) -> None:
        if self._sensorgroup != new_value:

            # Report new values
            try:
                ZoneSensorGroup(new_value)
            except ValueError:
                LOGGER.exception("Unknown Sensor group: %s, please report", new_value)

            self._sensorgroup = new_value
            LOGGER.debug("Zone%s (%s) - sensorgroup: %s", self.zone_id, self.sensorname, new_value.name)
            self.notify()

    @partition_id.setter
    def partition_id(self, value: str) -> None:
        if self.partition_id != value:
            self.partition_id = value
            self.notify()

    def to_dict(self) -> dict:
        return {
            "_id": self.id,
            "sensorid": self._sensor_id,
            "sensorname": self.sensorname,
            "sensorgroup": self.sensorgroup,
            "sensorstatus": self.sensorstatus.value,
            "sensorstate": self.sensorstate,
            "sensortype": self.sensortype.value,
            "zoneid": self.zone_id,
            "zone_type": self.zone_type,
            "zone_physical_type": self.zone_physical_type,
            "zone_alarm_type": self.zone_alarm_type,
            "partition_id": self.partition_id,
            "battery_status": self.battery_status,
            "sensortts": self._sensortts,
            "latestdBm": self.latestdBm,
            "averagedBm": self.averagedBm,
            "current_capability": self._current_capability,
            "zone_rf_sensor": self._zone_rf_sensor,
            "zone_supervised": self._zone_supervised,
            "zone_reporting_enabled": self._zone_reporting_enabled,
            "zone_two_way_voice_enabled": self._zone_two_way_voice_enabled,
            "signal_source": self._signal_source,
            "serial_number": self._serial_number,
            "chimetype": self._chimetype,
            "frame_count": self._frame_count,
            "frame_type": self._frame_type,
            "frame_id": self._frame_id,
            "allowdisarming": self._allowdisarming,
            "time": self.time,
            "version": self._version,
            "opr": self._opr,
            "zone_equipment_code": self._zone_equipement_code,
            "created_date": self._created_date,
            "created_by": self._created_by,
            "updated_by": self._updated_by,
            "updated_date": self._updated_date,
            "shortID": self._shortID,
            "diag_24hr": self._diag_24hr,
            "device_capability": self._device_capability,
            "sub_type": self._sub_type,
            "powerg_manufacture_id": self._powerg_manufacture_id,
            "parent_node": self._parent_node,
            "extras": self._extras,
            "ac_status": self._ac_status,
        }



import logging

LOGGER = logging.getLogger(__name__)


class QolsysError(Exception):

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)


class QolsysSslError(QolsysError):

    def __init__(self) -> None:
        super().__init__("QolsysSslError")


class QolsysMqttError(QolsysError):

    def __init__(self) -> None:
        super().__init__("QolsysMqttError")


class QolsysSqlError(QolsysError):

    def __init__(self, operation: dict) -> None:
        super().__init__("QolsysSqlError")

        table = f"QolsysSqlError - table:{operation.get('table', "")}"
        query = f"QolsysSqlError - query:{operation.get('query', "")}"
        columns = f"QolsysSqlError - columns:{operation.get('columns', "")}"
        content_values = f"QolsysSqlError - content_values:{operation.get('content_value', "")}"
        selection = f"QolsysSqlError - selection:{operation.get('selection', "")}"
        selection_argument = f"QolsysSqlError - selection_argument:{operation.get('selection_argument', "")}"

        e = f"""\n{table}\n{query}\n{columns}\n{content_values}\n{selection}\n{selection_argument}"""
        LOGGER.exception(e)



import logging
import os
import re
from datetime import datetime, timedelta
from pathlib import Path

from cryptography import x509
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.x509 import load_pem_x509_csr
from cryptography.x509.oid import NameOID

from .settings import QolsysSettings

LOGGER = logging.getLogger(__name__)


class QolsysPKI:
    def __init__(self, settings: QolsysSettings) -> None:
        self._id = ""
        self._subkeys_directory: Path = Path

        self._key = None
        self._cer = None
        self._csr = None
        self._secure = None
        self._qolsys = None

        self._settings = settings

    @property
    def id(self) -> str:
        return self._id

    def formatted_id(self) -> str:
        return ":".join(self.id[i:i+2] for i in range(0, len(self.id), 2))

    def set_id(self, pki_id: str) -> None:
        self._id = pki_id.replace(":", "").lower()
        LOGGER.debug("Using PKI: %s", self.formatted_id())
        self._subkeys_directory = self._settings.pki_directory.joinpath(Path(self.id))

    @property
    def key(self) -> str:
        return self._key

    @property
    def cer(self) -> str:
        return self._cer

    @property
    def csr(self) -> str:
        return self._csr

    @property
    def secure(self) -> str:
        return self._secure

    @property
    def qolsys(self) -> str:
        return self._qolsys

    def auto_discover_pki(self) -> bool:
        pattern = r"^[A-Fa-f0-9]{12}$"

        LOGGER.debug("Auto Discovery Enabled")
        with os.scandir(self._settings.pki_directory) as entries:
            for entry in entries:
                if entry.is_dir() and re.fullmatch(pattern, entry.name):
                    self.set_id(entry.name)
                    return True

        return False

    def load_private_key(self, key: str) -> bool:
        try:
            self._key = serialization.load_pem_private_key(key.encode(), password=None)
        except ValueError:
            LOGGER.debug("Private Key Value Error")
            return False

        return True

    def load_certificate(self, cer: str) -> bool:
        try:
            self._cer = x509.load_pem_x509_certificate(cer.encode(), None)
        except ValueError:
            LOGGER.debug("Certificate Value Error")
            return False

        return True

    def load_certificate_signing_request(self, csr: str) -> bool:
        try:
            self._csr = load_pem_x509_csr(csr.encode())
        except ValueError:
            LOGGER.debug("Certificate Signing Request Value Error")
            return False

        return True

    def load_qolsys_certificate(self, qolsys: str) -> bool:
        try:
            self._qolsys = x509.load_pem_x509_certificate(qolsys.encode(), None)
        except ValueError:
            LOGGER.debug("Qolsys Certificate Value Error")
            return False

        return True

    def load_signed_client_certificate(self, secure: str) -> bool:
        try:
            self._secure = x509.load_pem_x509_certificate(secure.encode(), None)
        except ValueError:
            LOGGER.debug("Client Signed Certificate Value Error")
            return False

        return True

    def check_key_file(self) -> bool:
        if self._subkeys_directory.joinpath(self.id + ".key").resolve().exists():
            LOGGER.debug("Found KEY")
            return True
        LOGGER.debug("No KEY File")
        return False

    def check_cer_file(self) -> bool:
        if self._subkeys_directory.joinpath(self.id + ".cer").resolve().exists():
            LOGGER.debug("Found CER")
            return True
        LOGGER.debug("No CER File")
        return False

    def check_csr_file(self) -> bool:
        if self._subkeys_directory.joinpath(self.id + ".csr").resolve().exists():
            LOGGER.debug("Found CSR")
            return True
        LOGGER.debug("No CSR File")
        return False

    def check_secure_file(self) -> bool:
        if self._subkeys_directory.joinpath(self.id + ".secure").resolve().exists():
            LOGGER.debug("Found Signed Client Certificate")
            return True
        LOGGER.debug("No Signed Client Certificate File")
        return False

    def check_qolsys_cer_file(self) -> bool:
        if self._subkeys_directory.joinpath(self.id + ".qolsys").resolve().exists():
            LOGGER.debug("Found Qolsys Certificate")
            return True
        LOGGER.debug("No Qolsys Certificate File")
        return False

    @property
    def key_file_path(self) -> Path:
        return self._subkeys_directory.joinpath(self.id + ".key")

    @property
    def csr_file_path(self) -> str:
        return self._subkeys_directory.joinpath(self.id + ".csr")

    @property
    def cer_file_path(self) -> str:
        return self._subkeys_directory.joinpath(self.id + ".cer")

    @property
    def secure_file_path(self) -> str:
        return self._subkeys_directory.joinpath(self.id + ".secure")

    @property
    def qolsys_cer_file_path(self) -> str:
        return self._subkeys_directory.joinpath(self.id + ".qolsys")

    def create(self, mac: str, key_size: int) -> bool:

        self.set_id(mac)

        # Check if directory exist
        if self._subkeys_directory.resolve().exists():
            LOGGER.error("Create Directory Colision")
            return False

        # Check for private key colision
        if self._subkeys_directory.joinpath(self.id + ".key").resolve().exists():
            LOGGER.error("Create KEY File Colision")
            return False

        # Check for CER file colision
        if self._subkeys_directory.joinpath(self.id + ".cer").resolve().exists():
            LOGGER.error("Create CER File Colision")
            return False

        # Check for CSR file colision
        if self._subkeys_directory.joinpath(self.id + ".csr").resolve().exists():
            LOGGER.error("Create CSR File Colision")
            return False

        # Check for CER file colision
        if self._subkeys_directory.joinpath(self.id + ".secure").resolve().exists():
            LOGGER.error("Create Signed Certificate File Colision")
            return False

        LOGGER.debug("Creating PKI:  %s", self.formatted_id())

        LOGGER.debug("Creating PKI Directory")
        self._subkeys_directory.resolve().mkdir(parents=True)

        LOGGER.debug("Creating KEY")
        private_key = rsa.generate_private_key(public_exponent=65537,
                                               key_size=key_size)
        private_pem = private_key.private_bytes(encoding=serialization.Encoding.PEM,
                                                format=serialization.PrivateFormat.PKCS8,
                                                encryption_algorithm=serialization.NoEncryption())
        with self._subkeys_directory.joinpath(self.id + ".key").open("wb") as file:
            file.write(private_pem)

        LOGGER.debug("Creating CER")
        subject = issuer = x509.Name([
            x509.NameAttribute(NameOID.COUNTRY_NAME, "US"),
            x509.NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, "SanJose"),
            x509.NameAttribute(NameOID.LOCALITY_NAME, ""),
            x509.NameAttribute(NameOID.ORGANIZATION_NAME, "Qolsys Inc."),
            x509.NameAttribute(NameOID.COMMON_NAME, "www.qolsys.com "),
        ])
        cert = x509.CertificateBuilder().subject_name(
            subject,
        ).issuer_name(
            issuer,
        ).public_key(
            private_key.public_key(),
        ).serial_number(
            x509.random_serial_number(),
        ).not_valid_before(
            datetime.utcnow(),
        ).not_valid_after(
            datetime.utcnow() + timedelta(days=365),
        ).add_extension(
            x509.BasicConstraints(ca=False, path_length=None), critical=True,
        ).sign(private_key, hashes.SHA256())
        cert_pem = cert.public_bytes(encoding=serialization.Encoding.PEM)

        with self._subkeys_directory.joinpath(self.id + ".cer").open("wb") as file:
            file.write(cert_pem)

        LOGGER.debug("Creating CSR")
        csr = x509.CertificateSigningRequestBuilder().subject_name(
            subject,
        ).add_extension(
            x509.BasicConstraints(ca=False, path_length=None), critical=True,
        ).sign(private_key, hashes.SHA256())

        # Save CSR to file
        csr_pem = csr.public_bytes(encoding=serialization.Encoding.PEM)
        with self._subkeys_directory.joinpath(self.id + ".csr").open("wb") as file:
            file.write(csr_pem)

        return True



import logging
import random
import re

LOGGER = logging.getLogger(__name__)


def fix_json_string(json_string):
    def replace_escape(match):
        hex_value = match.group(1)
        decimal_value = int(hex_value, 16)
        return f"\\u{decimal_value:04x}"

    return re.sub(r"\\x([0-9a-fA-F]{2})", replace_escape, json_string)


def generate_random_mac() -> str:
    mac = [
        0xf2, 0x16, 0x3e,
        random.randint(0x00, 0x7f),
        random.randint(0x00, 0xff),
        random.randint(0x00, 0xff),
    ]
    return ":".join(map(lambda x: "%02x" % x, mac))



import logging

from .observable import QolsysObservable
from .panel import QolsysPanel
from .settings import QolsysSettings
from .state import QolsysState

LOGGER = logging.getLogger(__name__)


class QolsysPlugin:
    def __init__(self, state: QolsysState, panel: QolsysPanel, settings: QolsysSettings) -> None:

        self._state: QolsysState = state
        self._panel: QolsysPanel = panel
        self._settings: QolsysSettings = settings

        self.connected = False
        self.connected_observer = QolsysObservable()

    def config(self) -> None:
        pass

    @property
    def state(self) -> QolsysState:
        return self._state

    @property
    def panel(self) -> QolsysPanel:
        return self._panel

    @property
    def settings(self) -> QolsysSettings:
        return self._settings



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableScene(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.SceneContentProvider/scene"
        self._table = "scene"
        self._abort_on_error = False

        self._columns = [
            "_id",
            "version",
            "opr",
            "partition_id",
            "scene_id",
            "name",
            "icon",
            "color",
            "flags",
            "ack",
            "create_time",
            "created_by",
            "update_time",
            "updated_by",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,version,opr,partition_id,scene_id,name,icon,
                             color,flags,ack,create_time,created_by,update_time,updated_by)
                              VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("version", ""),
            data.get("opr", ""),
            data.get("partition_id", ""),
            data.get("scene_id", ""),
            data.get("name", ""),
            data.get("icon", ""),
            data.get("color", ""),
            data.get("flags", ""),
            data.get("ack", ""),
            data.get("create_time", ""),
            data.get("created_by", ""),
            data.get("update_time", ""),
            data.get("updated_by", "")))

        self._db.commit()



import logging

from .zwave_device import QolsysZWaveDevice

LOGGER = logging.getLogger(__name__)


class QolsysLock(QolsysZWaveDevice):

    LOCK_STATUS_ARRAY = ["Locked"]  # noqa: RUF012

    def __init__(self, lock_dict: dict, zwave_dict: dict) -> None:

        super().__init__(zwave_dict)

        self._lock_id = lock_dict.get("_id")
        self._lock_version = lock_dict.get("version", "")
        self._lock_opr = lock_dict.get("opr", "")
        self._lock_partition_id = lock_dict.get("partition_id", "")
        self._lock_name = lock_dict.get("doorlock_name", "")
        self._lock_status = lock_dict.get("status", "")
        self._lock_node_id = lock_dict.get("node_id", "")
        self._lock_created_by = lock_dict.get("created_by", "")
        self._lock_created_date = lock_dict.get("created_date", "")
        self._lock_updated_by = lock_dict.get("updated_by", "")
        self._lock_last_updated_date = lock_dict.get("last_updated_date", "")
        self._lock_remote_arming = lock_dict.get("remote_arming", "")
        self._lock_keyfob_arming = lock_dict.get("keyfob_arming", "")
        self._lock_panel_arming = lock_dict.get("panel_arming", "")
        self._lock_endpoint = lock_dict.get("endpoint", "")
        self._lock_paired_status = lock_dict.get("paired_status", "")

    @property
    def lock_node_id(self) -> str:
        return self._lock_node_id

    @property
    def lock_status(self) -> str:
        return self._lock_status

    @property
    def lock_name(self) -> str:
        return self._lock_name

    @property
    def paired_status(self) -> str:
        return self._paired_status

    @lock_status.setter
    def lock_status(self, value: str) -> None:
        if self._lock_status != value:
            LOGGER.debug("Lock%s (%s) - status: %s", self.node_id, self.lock_name, value)
            self._lock_status = value
            self.notify()

    @lock_name.setter
    def lock_name(self, value: str) -> None:
        if self._lock_name != value:
            LOGGER.debug("Lock%s (%s) - name: %s", self.node_id, self.lock_name, value)
            self._lock_name = value
            self.notify()

    @paired_status.setter
    def paired_status(self, value: str) -> None:
        if self._paired_status != value:
            LOGGER.debug("Lock%s (%s) - paired_status: %s", self.node_id, self.lock_name, value)
            self._lock_paired_status = value
            self.notify()

    def update_lock(self, data: dict) -> None:  # noqa: C901, PLR0912
        # Check if we are updating same zoneid
        node_id_update = data.get("node_id", "")
        if node_id_update != self.lock_node_id:
            LOGGER.error(
                "Updating Lock %s (%s) with Lock '%s' (different id)", self.lock_node_id, self.lock_name, node_id_update)
            return

        self.start_batch_update()

        if "version" in data:
            self._lock_version = data.get("version")
        if "opr" in data:
            self._lock_opr = data.get("opr")
        if "partition_id" in data:
            self._lock_partition_id = data.get("partition_id")
        if "lock_name" in data:
            self.lock_name = data.get("lock_name")
        if "status" in data:
            self.lock_status = data.get("status")
        if "created_by" in data:
            self._lock_created_by = data.get("created_by")
        if "created_date" in data:
            self._lock_created_date = data.get("created_date")
        if "updated_by" in data:
            self._lock_updated_by = data.get("updated_by")
        if "last_updated_date" in data:
            self._lock_last_updated_date = data.get("last_updated_date")
        if "remote_arming" in data:
            self._lock_remote_arming = data.get("remote_arming")
        if "keyfob_arming" in data:
            self._lock_keyfob_arming = data.get("keyfob_arming")
        if "panel_arming" in data:
            self._lock_panel_arming = data.get("panel_arming")
        if "endpoint" in data:
            self._lock_endpoint = data.get("endpoint")
        if "paired_status" in data:
            self._lock_paired_status = data.get("paired_status")

        self.end_batch_update()

    def to_dict_dimmer(self) -> dict:
        return {
            "_id": self._lock_id,
            "version": self._lock_version,
            "opr": self._lock_opr,
            "partition_id": self._lock_partition_id,
            "doorlock_name": self.lock_name,
            "status": self.lock_status,
            "created_by": self._lock_created_by,
            "created_date": self._lock_created_date,
            "updated_by": self._lock_updated_by,
            "last_updated_date": self._lock_last_updated_date,
            "remote_arming": self._lock_remote_arming,
            "keyfob_arming": self._lock_keyfob_arming,
            "panel_arming": self._lock_panel_arming,
            "endpoint": self._lock_endpoint,
            "paired_status": self._lock_paired_status,
        }



import logging

from .enum import (
    PartitionAlarmState,
    PartitionAlarmType,
    PartitionSystemStatus,
)
from .observable import QolsysObservable

LOGGER = logging.getLogger(__name__)


class QolsysPartition(QolsysObservable):

    EXIT_SOUNDS_ARRAY = ["ON", "OFF", ""]  # noqa: RUF012
    ENTRY_DELAYS_ARRAY = ["ON", "OFF", ""]  # noqa: RUF012

    def __init__(self, partition_dict: dict, settings_dict: dict, alarm_state: PartitionAlarmState,
                 alarm_type_array: list[PartitionAlarmType]) -> None:

        super().__init__()

        # Partition info (partition table)
        self._id: str = partition_dict.get("partition_id")
        self._name: str = partition_dict.get("name")
        self._devices = partition_dict.get("devices")

        # Partition Settings (qolsyssettings table)
        self._system_status: PartitionSystemStatus = PartitionSystemStatus(settings_dict.get("SYSTEM_STATUS", ""))
        self._system_status_changed_time: str = settings_dict.get("SYSTEM_STATUS_CHANGED_TIME", "")
        self._exit_sounds: str = settings_dict.get("EXIT_SOUNDS", "")
        self._entry_delays: str = settings_dict.get("ENTRY_DELAYS", "")

        # Alarm State (state table)
        self._alarm_state: PartitionAlarmState = alarm_state

        # Alarm Type (alarmedsensor table)
        self._alarm_type_array: list[PartitionAlarmType] = alarm_type_array

        # Other
        self._command_exit_sounds = True
        self._command_silent_disarming = False
        self._command_arm_stay_instant = True

    @property
    def id(self) -> int:
        return self._id

    @property
    def name(self) -> str:
        return self._name

    @property
    def system_status(self) -> PartitionSystemStatus:
        return self._system_status

    @property
    def system_status_changed_time(self) -> str:
        return self._system_status_changed_time

    @property
    def alarm_state(self) -> PartitionAlarmState:
        return self._alarm_state

    @property
    def alarm_type_array(self) -> list[PartitionAlarmType]:
        return self._alarm_type_array

    @property
    def exit_sounds(self) -> str:
        return self._exit_sounds

    @property
    def entry_delays(self) -> str:
        return self._entry_delays

    @property
    def command_exit_sounds(self) -> bool:
        return self._command_exit_sounds

    @property
    def command_arm_stay_instant(self) -> bool:
        return self._command_arm_stay_instant

    @property
    def command_silent_disarming(self) -> bool:
        return self._command_silent_disarming

    @command_silent_disarming.setter
    def command_silent_disarming(self, value: bool) -> None:
        self._command_silent_disarming = value

    @system_status.setter
    def system_status(self, new_value: PartitionSystemStatus) -> None:
        if self._system_status != new_value:
            LOGGER.debug("Partition%s (%s) - system_status: %s", self.id, self.name, new_value)
            self._system_status = new_value
            self.notify()

    @system_status_changed_time.setter
    def system_status_changed_time(self, value: str) -> None:
        if self._system_status_changed_time != value:
            LOGGER.debug("Partition%s (%s) - system_status_changed_time: %s", self._id, self._name, value)
            self._system_status_changed_time = value
            self.notify()

    @alarm_state.setter
    def alarm_state(self, new_value: PartitionAlarmState) -> None:
        if self._alarm_state != new_value:
            LOGGER.debug("Partition%s (%s) - alarm_state: %s", self.id, self.name, new_value)
            self._alarm_state = new_value
            self.notify()

    @alarm_type_array.setter
    def alarm_type_array(self, new_alarm_type_array: list[PartitionAlarmType]) -> None:

        # If no changes are detected: return without notification
        if sorted(new_alarm_type_array, key=lambda c: c.value) == sorted(self.alarm_type_array, key=lambda c: c.value):
            return

        # alarm_type_array are different:
        self._alarm_type_array = []

        # If all alarm have been cleared
        if new_alarm_type_array == []:
            LOGGER.debug("Partition%s (%s) - alarm_type: %s", self._id, self._name, "None")
            self.notify()
            return

        self.append_alarm_type(new_alarm_type_array)

    @name.setter
    def name(self, value: str) -> None:
        if self._name != value:
            LOGGER.debug("Partition%s (%s) - name: %s", self._id, self._name, value)
            self._name = value
            self.notify()

    def append_alarm_type(self, new_alarm_type_array: list[PartitionAlarmType]) -> None:

        data_changed = False

        for new_alarm_type in new_alarm_type_array:

            # Value already in array
            if new_alarm_type in self._alarm_type_array:
                continue

            self._alarm_type_array.append(new_alarm_type)
            data_changed = True

        if data_changed:
            self.notify()
            for alarm in self._alarm_type_array:
                LOGGER.debug("Partition%s (%s) - alarm_type: %s", self._id, self._name, alarm)

    @exit_sounds.setter
    def exit_sounds(self, value: str) -> None:
        if value not in self.EXIT_SOUNDS_ARRAY:
            LOGGER.debug("Partition%s (%s) - Unknow exit_sounds %s", self._id, self._name, value)
            return

        if self._exit_sounds != value:
            LOGGER.debug("Partition%s (%s) - exit_sound: %s", self._id, self._name, value)
            self._exit_sounds = value
            self.notify()

    @entry_delays.setter
    def entry_delays(self, value: str) -> None:
        if value not in self.ENTRY_DELAYS_ARRAY:
            LOGGER.debug("Partition%s (%s) - Unknow entry_delays %s", self._id, self._name, value)
            return

        if self._entry_delays != value:
            LOGGER.debug("Partition%s (%s) - entry_delays: %s", self._id, self._name, value)
            self._entry_delays = value
            self.notify()

    @command_exit_sounds.setter
    def command_exit_sounds(self, value: str) -> None:
        self._command_exit_sounds = value
        LOGGER.debug("Partition%s (%s) - command_exit_sounds: %s", self._id, self._name, value)
        self.notify()

    @command_arm_stay_instant.setter
    def command_arm_stay_instant(self, value: str) -> None:
        self._command_arm_stay_instant = value
        LOGGER.debug("Partition%s (%s) - arm_stay_instant: %s", self._id, self._name, value)
        self.notify()

    def update_partition(self, data: dict) -> None:
        # Check if we are updating same partition_id
        partition_id_update = data.get("partition_id", "")
        if int(partition_id_update) != int(self.id):
            LOGGER.error(
                "Updating Partition%s (%s) with Partition '%s' (different id)", self._id, self._name, partition_id_update)
            return

        self.start_batch_update()

        # Update Partition Name
        if "name" in data:
            self.name = data.get("name")

        # Update Partition Devices
        if "devices" in data:
            self._devices = data.get("devices")

        self.end_batch_update()

    def update_settings(self, data: dict) -> None:

        self.start_batch_update()

        # Update system_status
        if "SYSTEM_STATUS" in data:
            self.system_status = data.get("SYSTEM_STATUS")

        # Update system_status_changed_time
        if "SYSTEM_STATUS_CHANGED_TIME" in data:
            self.system_status_changed_time = data.get("SYSTEM_STATUS_CHANGED_TIME")

        # Update exit_sounds
        if "EXIT_SOUNDS" in data:
            self.exit_sounds = data.get("EXIT_SOUNDS")

        # Update entry_delays
        if "ENTRY_DELAYS" in data:
            self.entry_delays = data.get("ENTRY_DELAYS")

        self.end_batch_update()

    def to_dict_partition(self) -> dict:
        return {
            "partition_id": self.id,
            "name": self.name,
            "devices": self._devices,
        }

    def to_dict_settings(self) -> dict:
        return {
            "SYSTEM_STATUS": self.system_status.value,
            "SYSTEM_STATUS_CHANGED_TIME": self.system_status_changed_time,
            "EXIT_SOUNDS": self.exit_sounds,
            "ENTRY_DELAYS": self.entry_delays,
        }



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableThermostat(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.ThermostatsContentProvider/thermostat"
        self._table = "thermostat"
        self._abort_on_error = False

        self._columns = [
            "_id",
            "version",
            "opr",
            "partition_id",
            "thermostat_id",
            "thermostat_name",
            "current_temp",
            "target_cool_temp",
            "target_heat_temp",
            "target_temp",
            "power_usage",
            "thermostat_mode",
            "thermostat_mode_bitmask",
            "fan_mode",
            "fan_mode_bitmask",
            "set_point_mode",
            "set_point_mode_bitmask",
            "node_id",
            "created_by",
            "created_date",
            "updated_by",
            "last_updated_date",
            "thermostat_mode_updated_time",
            "fan_mode_updated_time",
            "set_point_mode_updated_time",
            "target_cool_temp_updated_time",
            "target_heat_temp_updated_time",
            "current_temp_updated_time",
            "device_temp_unit",
            "endpoint",
            "paired_status",
            "configuration_parameter",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,version,opr,partition_id,thermostat_id,thermostat_name,
                             current_temp,target_cool_temp,target_heat_temp,target_temp,power_usage,thermostat_mode,
                             thermostat_mode_bitmask,fan_mode,fan_mode_bitmask,set_point_mode,set_point_mode_bitmask,
                             node_id,created_by,created_date,updated_by,last_updated_date,thermostat_mode_updated_time,
                             fan_mode_updated_time,set_point_mode_updated_time,target_cool_temp_updated_time,
                             target_heat_temp_updated_time,current_temp_updated_time,device_temp_unit,endpoint,
                             paired_status,configuration_parameter)
                              VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("version", ""),
            data.get("opr", ""),
            data.get("partition_id", ""),
            data.get("thermostat_id", ""),
            data.get("thermostat_name", ""),
            data.get("current_temp", ""),
            data.get("target_cool_temp", ""),
            data.get("target_heat_temp", ""),
            data.get("target_temp", ""),
            data.get("power_usage", ""),
            data.get("thermostat_mode", ""),
            data.get("thermostat_mode_bitmask,fan_mode", ""),
            data.get("fan_mode_bitmask,set_point_mode", ""),
            data.get("set_point_mode_bitmask", ""),
            data.get("node_id", ""),
            data.get("created_by", ""),
            data.get("created_date", ""),
            data.get("updated_by", ""),
            data.get("last_updated_date", ""),
            data.get("thermostat_mode_updated_time", ""),
            data.get("fan_mode_updated_time", ""),
            data.get("set_point_mode_updated_time", ""),
            data.get("target_cool_temp_updated_time", ""),
            data.get("target_heat_temp_updated_time", ""),
            data.get("current_temp_updated_time", ""),
            data.get("device_temp_unit", ""),
            data.get("endpoint", ""),
            data.get("paired_status", ""),
            data.get("configuration_parameter", "")))

        self._db.commit()



import logging

from .zwave_device import QolsysZWaveDevice

LOGGER = logging.getLogger(__name__)


class QolsysDimmer(QolsysZWaveDevice):

    def __init__(self, dimmer_dict: dict, zwave_dict: dict) -> None:

        super().__init__(zwave_dict)

        self._dimmer_id = dimmer_dict.get("_id")
        self._dimmer_version = dimmer_dict.get("version", "")
        self._dimmer_opr = dimmer_dict.get("opr", "")
        self._dimmer_partition_id = dimmer_dict.get("partition_id", "")
        self._dimmer_name = dimmer_dict.get("dimmer_name", "")
        self._dimmer_status = dimmer_dict.get("status", "")
        self._dimmer_level = dimmer_dict.get("level", "")
        self._dimmer_node_id = dimmer_dict.get("node_id", "")
        self._dimmer_created_by = dimmer_dict.get("created_by", "")
        self._dimmer_created_date = dimmer_dict.get("created_date", "")
        self._dimmer_updated_by = dimmer_dict.get("updated_by", "")
        self._dimmer_last_updated_date = dimmer_dict.get("last_updated_date", "")
        self._dimmer_endpoint = dimmer_dict.get("endpoint", "")
        self._dimmer_power_details = dimmer_dict.get("power_details", "")
        self._dimmer_paired_status = dimmer_dict.get("paired_status", "")

    @property
    def dimmer_node_id(self) -> str:
        return self._dimmer_node_id

    @property
    def dimmer_status(self) -> str:
        return self._dimmer_status

    @property
    def dimmer_name(self) -> str:
        return self._dimmer_name

    @property
    def dimmer_level(self) -> str:
        return self._dimmer_level

    @dimmer_level.setter
    def dimmer_level(self, value: str) -> None:
        if self._dimmer_level != value:
            LOGGER.debug("Dimmer%s (%s) - level: %s", self.node_id, self.dimmer_name, value)
            self._dimmer_level = value
            self.notify()

    @dimmer_status.setter
    def dimmer_status(self, value: str) -> None:
        if self._dimmer_status != value:
            LOGGER.debug("Dimmer%s (%s) - status: %s", self.node_id, self.dimmer_name, value)
            self._dimmer_status = value
            self.notify()

    @dimmer_name.setter
    def dimmer_name(self, value: str) -> None:
        if self._dimmer_name != value:
            LOGGER.debug("Dimmer%s (%s) - name: %s", self.node_id, self.dimmer_name, value)
            self._dimmer_name = value
            self.notify()

    def is_on(self) -> bool:
        return self.dimmer_status == "On"

    def update_dimmer(self, content_values: dict) -> None:  # noqa: C901
        # Check if we are updating same none_id
        node_id_update = content_values.get("node_id", "")
        if node_id_update != self._dimmer_node_id:
            LOGGER.error(
                "Updating Dimmer %s (%s) with dimmer %s (different id)", self._node_id, self.dimmer_name, node_id_update)
            return

        self.start_batch_update()

        if "status" in content_values:
            self.dimmer_status = content_values.get("status")
        if "level" in content_values:
            self.dimmer_level = content_values.get("level")
        if "dimmer_name" in content_values:
            self.dimmer_name = content_values.get("dimmer_name")
        if "created_by" in content_values:
            self._dimmer_created_by = content_values.get("created_by")
        if "created_date" in content_values:
            self._dimmer_created_date = content_values.get("created_date")
        if "version" in content_values:
            self._dimmer_version = content_values.get("version")
        if "opr" in content_values:
            self._dimmer_opr = content_values.get("opr")
        if "partition_id" in content_values:
            self.partition_id = content_values.get("partition_id")
        if "updated_by" in content_values:
            self._dimmer_updated_by = content_values.get("updated_by")
        if "last_updated_date" in content_values:
            self._last_updated_date = content_values.get("last_updated_date")
        if "endpoint" in content_values:
            self._dimmer_endpoint = content_values.get("endpoint")
        if "power_details" in content_values:
            self._dimmer_power_details = content_values.get("power_details")
        if "paired_status" in content_values:
            self._dimmer_paired_status = content_values.get("paired_status")

        self.end_batch_update()

    def to_dict_dimmer(self) -> dict:
        return {
            "_id": self._dimmer_id,
            "version": self._dimmer_version,
            "opr": self._dimmer_opr,
            "partition_id": self._partition_id,
            "dimmer_name": self.dimmer_name,
            "status": self.dimmer_status,
            "level": self.dimmer_level,
            "node_id": self.dimmer_node_id,
            "created_by": self._dimmer_created_by,
            "created_date": self._dimmer_created_date,
            "updated_by": self._dimmer_updated_by,
            "last_updated_date": self._last_updated_date,
            "endpoint": self._dimmer_endpoint,
            "power_details": self._dimmer_power_details,
            "paired_status": self._dimmer_paired_status,
        }



import socket

from zeroconf import ServiceInfo
from zeroconf.asyncio import AsyncZeroconf


class QolsysMDNS:

    def __init__(self, ip: str, port: int) -> None:

        self.azc = AsyncZeroconf()

        self.mdns_info = ServiceInfo(
            "_http._tcp.local.",
            "NsdPairService._http._tcp.local.",
            addresses=[socket.inet_aton(ip)],
            port=port,
        )

    async def start_mdns(self) -> None:
        await self.azc.async_register_service(self.mdns_info)

    async def stop_mdns(self) -> None:
        await self.azc.async_unregister_service(self.mdns_info)



import logging

from .plugin import QolsysPlugin

LOGGER = logging.getLogger(__name__)


class QolsysPluginC4(QolsysPlugin):
    def __init__(self) -> None:

        # C4 Integration
        self._token = ""

    def config(self, panel_ip: str, token: str) -> bool:  # noqa: ARG002
        LOGGER.warning("C4Plugin: Configuring Plugin")
        super().config()
        return True



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableVirtualDevice(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.VirtualDeviceContentProvider/virtual_device"
        self._table = "virtual_device"
        self._abort_on_error = False

        self._columns = [
            "_id",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        if data is not None:
            LOGGER.error("Please Report")
            LOGGER.error("Loading Table Format: %s", self.uri)
            LOGGER.error(data)



import asyncio
import logging
from collections.abc import Coroutine

LOGGER = logging.getLogger(__name__)


class QolsysTaskManager:
    def __init__(self) -> None:
        self._tasks = set()

    def run(self, coro: Coroutine, label: str) -> asyncio.Task:
        task = asyncio.create_task(coro, name=label)
        self._tasks.add(task)

        def _done_callback(task: asyncio.Task) -> None:
            self._tasks.discard(task)

        task.add_done_callback(_done_callback)
        return task

    def cancel(self, label: str) -> None:
        for task in self._tasks:
            if task.get_name() == label:
                task.cancel()

    def cancel_all(self) -> None:
        for task in self._tasks:
            task.cancel()

    async def wait_all(self) -> None:
        if self._tasks:
            await asyncio.gather(*self._tasks, return_exceptions=True)

    def pending(self) -> None:
        return {t for t in self._tasks if not t.done()}

    def dump(self) -> None:
        for task in self._tasks:
            LOGGER.debug("Task: %s, Done: %s, Cancelled: %s", task.get_name(), task.done(), task.cancelled())



import asyncio
import datetime
import json
import logging
import random
import ssl
import uuid

import aiomqtt

from .enum import PartitionAlarmState, PartitionSystemStatus
from .errors import QolsysMqttError, QolsysSslError
from .mdns import QolsysMDNS
from .mqtt_command_queue import QolsysMqttCommandQueue
from .panel import QolsysPanel
from .pki import QolsysPKI
from .plugin import QolsysPlugin
from .settings import QolsysSettings
from .state import QolsysState
from .task_manager import QolsysTaskManager
from .utils_mqtt import generate_random_mac

LOGGER = logging.getLogger(__name__)


class QolsysPluginRemote(QolsysPlugin):

    def __init__(self, state: QolsysState, panel: QolsysPanel, settings: QolsysSettings) -> None:
        super().__init__(state, panel, settings)

        # PKI
        self._pki = QolsysPKI(settings=settings)
        self._auto_discover_pki = True

        # Plugin
        self.certificate_exchange_server = None
        self._check_user_code_on_disarm = True
        self._log_mqtt_messages = False
        self._task_manager = QolsysTaskManager()
        self._mqtt_command_queue = QolsysMqttCommandQueue()

        # MQTT Client
        self.aiomqtt = None
        self._mqtt_task_config_label = "mqtt_task_config"
        self._mqtt_task_listen_label = "mqtt_task_listen"
        self._mqtt_task_connect_label = "mqtt_task_connect"
        self._mqtt_task_ping_label = "mqtt_task_ping"

    @property
    def log_mqtt_mesages(self) -> bool:
        return self._log_mqtt_messages

    @log_mqtt_mesages.setter
    def log_mqtt_mesages(self, log_mqtt_mesages: bool) -> None:
        self._log_mqtt_messages = log_mqtt_mesages

    @property
    def check_user_code_on_disarm(self) -> bool:
        return self._check_user_code_on_disarm

    @check_user_code_on_disarm.setter
    def check_user_code_on_disarm(self, check_user_code_on_disarm: bool) -> None:
        self._check_user_code_on_disarm = check_user_code_on_disarm

    @property
    def auto_discover_pki(self) -> bool:
        return self._auto_discover_pki

    @auto_discover_pki.setter
    def auto_discover_pki(self, value: bool) -> None:
        self._auto_discover_pki = value

    def is_paired(self) -> bool:
        # Check if plugin is paired:
        # 1- random_mac set
        # 2- KEY file present
        # 3- Signed certificate file present
        # 4- Qolsys certificate present
        # 5- Qolsys Panel IP present
        return (
            self._pki.id != "" and
            self._pki.check_key_file() and
            self._pki.check_cer_file() and
            self._pki.check_qolsys_cer_file() and
            self._pki.check_secure_file() and
            self.settings.check_panel_ip() and
            self.settings.check_plugin_ip()
        )

    async def config(self, start_pairing: bool) -> bool:
        return await self._task_manager.run(self.config_task(start_pairing), self._mqtt_task_config_label)

    async def config_task(self, start_pairing: bool) -> bool:
        LOGGER.debug("Configuring Plugin")
        super().config()

        # Check and created config_directory
        if not self.settings.check_config_directory(create=start_pairing):
            return False

        # Read user file for access code
        loop = asyncio.get_running_loop()
        if not loop.run_in_executor(None, self.panel.read_users_file):
            return False

        # Config PKI
        if self._auto_discover_pki:
            if self._pki.auto_discover_pki():
                self.settings.random_mac = self._pki.formatted_id()
        else:
            self._pki.set_id(self.settings.random_mac)

        # Set mqtt_remote_client_id
        self.settings.mqtt_remote_client_id = "qolsys-controller-" + self._pki.formatted_id()
        LOGGER.debug("Using MQTT remoteClientID: %s", self.settings.mqtt_remote_client_id)

        # Check if plugin is paired
        if self.is_paired():
            LOGGER.debug("Panel is Paired")

        else:
            LOGGER.debug("Panel not paired")

            if not start_pairing:
                LOGGER.debug("Aborting pairing.")
                return False

            if not await self.start_initial_pairing():
                LOGGER.debug("Error Pairing with Panel")
                return False

        LOGGER.debug("Starting Plugin Operation")

        # Everything is configured
        return True

    async def start_operation(self) -> None:
        await self._task_manager.run(self.mqtt_connect_task(reconnect=True), self._mqtt_task_connect_label)

    async def stop_operation(self) -> None:
        LOGGER.debug("Stopping Plugin Operation")

        if self.certificate_exchange_server is not None:
            self.certificate_exchange_server.close()

        if self.aiomqtt is not None:
            await self.aiomqtt.__aexit__(None, None, None)
            self.aiomqtt = None

        self._task_manager.cancel(self._mqtt_task_connect_label)
        self._task_manager.cancel(self._mqtt_task_listen_label)
        self._task_manager.cancel(self._mqtt_task_ping_label)
        self._task_manager.cancel(self._mqtt_task_config_label)

        self.connected = False
        self.connected_observer.notify()

    async def mqtt_connect_task(self, reconnect: bool) -> None:
        # Configure TLS parameters for MQTT connection
        tls_params = aiomqtt.TLSParameters(
            ca_certs=self._pki.qolsys_cer_file_path,
            certfile=self._pki.secure_file_path,
            keyfile=self._pki.key_file_path,
            cert_reqs=ssl.CERT_REQUIRED,
            tls_version=ssl.PROTOCOL_TLSv1_2,
            ciphers="ALL:@SECLEVEL=0",
        )

        LOGGER.debug("MQTT: Connecting ...")

        while True:
            try:
                self.aiomqtt = aiomqtt.Client(
                    hostname=self.settings.panel_ip,
                    port=8883,
                    tls_params=tls_params,
                    tls_insecure=True,
                    clean_session=True,
                    timeout=self.settings.mqtt_timeout,
                    identifier="QolsysController",
                )

                await self.aiomqtt.__aenter__()

                LOGGER.debug("MQTT: Client Connected")

                # Subscribe to panel internal databse updates
                await self.aiomqtt.subscribe("iq2meid")

                # Subscribte to MQTT commands response
                await self.aiomqtt.subscribe("response_" + self.settings.random_mac, qos=0)

                # Only log mastermeid traffic for debug purposes
                if self.log_mqtt_mesages:
                    # Subscribe to MQTT commands send to panel by other devices
                    await self.aiomqtt.subscribe("mastermeid", qos=0)

                    # Subscribe to all topics
                    #await self.aiomqtt.subscribe("#", qos=0)

                # Start mqtt_listent_task and mqtt_ping_task
                self._task_manager.cancel(self._mqtt_task_listen_label)
                self._task_manager.cancel(self._mqtt_task_ping_label)
                self._task_manager.run(self.mqtt_listen_task(), self._mqtt_task_listen_label)
                self._task_manager.run(self.mqtt_ping_task(), self._mqtt_task_ping_label)

                response_connect = await self.command_connect()
                self.panel.imei = response_connect.get("master_imei", "")
                self.panel.product_type = response_connect.get("primary_product_type", "")

                await self.command_pingevent()
                await self.command_timesync()
                await self.command_pair_status_request()

                response_database = await self.command_sync_database()
                LOGGER.debug("MQTT: Updating State from syncdatabase")
                self.panel.load_database(response_database.get("fulldbdata"))
                self.panel.dump()
                self.state.dump()

                self.connected = True
                self.connected_observer.notify()
                break

            except aiomqtt.MqttError as err:
                # Receive pannel network error
                self.connected = False
                self.connected_observer.notify()
                await self.aiomqtt.__aexit__(None, None, None)
                self.aiomqtt = None

                if reconnect:
                    LOGGER.debug("MQTT Error - %s: Reconnecting in %s seconds ...", err, self.settings.mqtt_timeout)
                    await asyncio.sleep(self.settings.mqtt_timeout)
                else:
                    raise QolsysMqttError from err

            except ssl.SSLError as err:
                # SSL error is and authentication error with invalid certificates en pki
                # We cannot recover from this error automaticly
                # Pannels need to be re-paired
                self.connected = False
                self.connected_observer.notify()
                await self.aiomqtt.__aexit__(None, None, None)
                self.aiomqtt = None
                raise QolsysSslError from err

    async def mqtt_ping_task(self) -> None:
        while True:
            if self.aiomqtt is not None and self.connected:
                await self.command_pingevent()

            await asyncio.sleep(self.settings.mqtt_ping)

    async def mqtt_listen_task(self) -> None:
        try:
            async for message in self.aiomqtt.messages:

                if self.log_mqtt_mesages:
                    LOGGER.debug("MQTT TOPIC: %s\n%s", message.topic, message.payload.decode())

                # Panel response to MQTT Commands
                if message.topic.matches("response_" + self.settings.random_mac):
                    data = message.payload.decode()
                    # data = message.payload.decode().replace("\\\\", "\\")
                    # data = fix_json_string(data)
                    data = json.loads(data)
                    await self._mqtt_command_queue.handle_response(data)

                # Panel updates to IQ2MEID database
                if message.topic.matches("iq2meid"):
                    data = json.loads(message.payload.decode())
                    self.panel.parse_iq2meid_message(data)

        except aiomqtt.MqttError as err:
            self.connected = False
            self.connected_observer.notify()

            LOGGER.debug("%s: Reconnecting in %s seconds ...", err, self.settings.mqtt_timeout)
            await asyncio.sleep(self.settings.mqtt_timeout)
            self._task_manager.run(self.mqtt_connect_task(reconnect=True), self._mqtt_task_connect_label)

        except ssl.SSLError as err:
            # SSL error is and authentication error with invalid certificates en pki
            # We cannot recover from this error automaticly
            # Pannels need to be re-paired
            self.connected = False
            self.connected_observer.notify()
            await self.aiomqtt.__aexit__(None, None, None)
            self.aiomqtt = None
            raise QolsysSslError from err

    async def start_initial_pairing(self) -> bool:
        # check if random_mac exist
        if self.settings.random_mac == "":
            LOGGER.debug("Creating random_mac")
            self.settings.random_mac = generate_random_mac()
            self._pki.create(self.settings.random_mac, key_size=self.settings.key_size)

        # Check if PKI is valid
        self._pki.set_id(self.settings.random_mac)
        LOGGER.debug("Checking PKI")
        if not (
            self._pki.check_key_file() and
            self._pki.check_cer_file() and
            self._pki.check_csr_file()
        ):
            LOGGER.error("PKI Error")
            return False

        LOGGER.debug("Starting Pairing Process")

        if not self.settings.check_plugin_ip():
            LOGGER.error("Plugin IP Address not configured")
            return False

        # If we dont allready have client signed certificate, start the pairing server
        if not self._pki.check_secure_file() or not self._pki.check_qolsys_cer_file() or not self.settings.check_panel_ip():

            # High Level Random Pairing Port
            pairing_port = random.randint(50000, 55000)

            # Start Pairing mDNS Brodcast
            LOGGER.debug("Starting mDNS Service Discovery: %s:%s", self.settings.plugin_ip, str(pairing_port))
            mdns_server = QolsysMDNS(self.settings.plugin_ip, pairing_port)
            await mdns_server.start_mdns()

            # Start Key Exchange Server
            LOGGER.debug("Starting Certificate Exchange Server")
            context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
            context.load_cert_chain(certfile=self._pki.cer_file_path, keyfile=self._pki.key_file_path)
            self.certificate_exchange_server = await asyncio.start_server(self.handle_key_exchange_client,
                                                                          self.settings.plugin_ip, pairing_port, ssl=context)

            LOGGER.debug("Certificate Exchange Server Waiting for Panel")
            LOGGER.debug("Press Pair Button in IQ Remote Config Page ...")

            async with self.certificate_exchange_server:
                try:
                    await self.certificate_exchange_server.serve_forever()

                except asyncio.CancelledError:
                    LOGGER.debug("Stoping Certificate Exchange Server")
                    LOGGER.debug("Stoping mDNS Service Discovery")
                    await mdns_server.stop_mdns()

        LOGGER.debug("Sending MQTT Pairing Request to Panel")

        # We have client sgined certificate at this point
        # Connect to Panel MQTT to send pairing command
        await self._task_manager.run(self.mqtt_connect_task(reconnect=False), self._mqtt_task_connect_label)
        LOGGER.debug("Plugin Pairing Completed ")
        return True

    async def handle_key_exchange_client(self, reader: asyncio.StreamReader, writer: asyncio.StreamWriter) -> None:

        received_panel_mac = False
        received_signed_client_certificate = False
        received_qolsys_cer = False

        signed_certificate_data = ""
        qolsys_certificate_data = ""

        try:
            continue_pairing = True
            while continue_pairing:

                # Plugin is receiving panel_mac from panel
                if (not received_panel_mac and not received_signed_client_certificate and not received_qolsys_cer):

                    request = (await reader.read(2048))
                    mac = request.decode()

                    address, port = writer.get_extra_info("peername")
                    LOGGER.debug("Panel Connected from: %s:%s", address, port)
                    LOGGER.debug("Receiving from Panel: %s", mac)

                    # Remove \x00 and \x01 from received string
                    self.settings.panel_mac = "".join(char for char in mac if char.isprintable())
                    self.settings.panel_ip = address
                    received_panel_mac = True

                    # Sending random_mac to panel
                    message = b"\x00\x11" + self.settings.random_mac.encode()
                    LOGGER.debug("Sending to Panel: %s", message.decode())
                    writer.write(message)
                    await writer.drain()

                    # Sending CSR File to panel
                    with open(self._pki.csr_file_path, "rb") as file:
                        content = file.read()
                        LOGGER.debug("Sending to Panel: [CSR File Content]")
                        writer.write(content)
                        writer.write(b"sent")
                        await writer.drain()

                    continue

                # Read signed certificate data
                if (received_panel_mac and not received_signed_client_certificate and not received_qolsys_cer):

                    request = (await reader.readline())
                    signed_certificate_data += request.decode()

                    if "sent" in request.decode():
                        certificates = [item for item in signed_certificate_data.split("sent") if item]
                        LOGGER.debug("Saving [Signed Client Certificate]")
                        with open(self._pki.secure_file_path, "wb") as f:
                            f.write(certificates[0].encode())
                            received_signed_client_certificate = True

                        if len(certificates) > 1:
                            qolsys_certificate_data += certificates[1]

                # Read qolsys certificate data
                if (received_panel_mac and received_signed_client_certificate and not received_qolsys_cer):

                    request = (await reader.readline())
                    qolsys_certificate_data += request.decode()

                    if "sent" in request.decode():
                        continue_pairing = False
                        certificates = [item for item in qolsys_certificate_data.split("sent") if item]

                        LOGGER.debug("Saving [Qolsys Certificate]")
                        with open(self._pki.qolsys_cer_file_path, "w") as f:
                            f.write(certificates[0])
                            received_qolsys_cer = True
                            continue_pairing = False
                            writer.close()

                    continue

        except asyncio.CancelledError:
            LOGGER.exception("Key Exchange Server asyncio CancelledError")

        except Exception:
            LOGGER.exception("Key Exchange Server error")

        finally:
            writer.close()
            self.certificate_exchange_server.close()

    async def send_command(self, topic: str, json_payload: str, request_id: str) -> dict:
        if self.aiomqtt is None:
            LOGGER.error("MQTT Client not configured")
            raise QolsysMqttError

        await self.aiomqtt.publish(topic=topic, payload=json.dumps(json_payload), qos=0)
        return await self._mqtt_command_queue.wait_for_response(request_id)

    async def command_connect(self) -> dict:
        LOGGER.debug("MQTT: Sending connect command")

        topic = "mastermeid"
        ipAddress = self.settings.plugin_ip
        eventName = "connect_v204"
        macAddress = self.settings.random_mac
        remoteClientID = self.settings.mqtt_remote_client_id
        softwareVersion = "4.4.1"
        producType = "tab07_rk68"
        bssid = ""
        lastUpdateChecksum = "2132501716"
        dealerIconsCheckSum = ""
        remote_feature_support_version = "1"
        current_battery_status = "Normal"
        remote_panel_battery_status = 3
        remote_panel_battery_health = 2
        remote_panel_battery_level = 100
        remote_panel_battery_present = True
        remote_panel_battery_percentage = 100
        remote_panel_battery_scale = 100
        remote_panel_battery_voltage = 4082
        remote_panel_battery_technology = ""
        remote_panel_plugged = 1
        remote_panel_battery_temperature = 430
        requestID = str(uuid.uuid4())
        responseTopic = "response_" + self.settings.random_mac
        remoteMacAddress = self.settings.random_mac

        dhcpInfo = {
            "ipaddress": "",
            "gateway": "",
            "netmask": "",
            "dns1": "",
            "dns2": "",
            "dhcpServer": "",
            "leaseDuration": "",
        }

        payload = {
            "eventName": eventName,
            "pairing_request": True,
            "ipAddress": ipAddress,
            "macAddress": macAddress,
            "remoteClientID": remoteClientID,
            "softwareVersion": softwareVersion,
            "productType": producType,
            "bssid": bssid,
            "dhcpInfo": json.dumps(dhcpInfo),
            "lastUpdateChecksum": lastUpdateChecksum,
            "dealerIconsCheckSum": dealerIconsCheckSum,
            "remote_feature_support_version": remote_feature_support_version,
            "current_battery_status": current_battery_status,
            "remote_panel_battery_status": remote_panel_battery_status,
            "remote_panel_battery_health": remote_panel_battery_health,
            "remote_panel_battery_level": remote_panel_battery_level,
            "remote_panel_battery_present": remote_panel_battery_present,
            "remote_panel_battery_percentage": remote_panel_battery_percentage,
            "remote_panel_battery_scale": remote_panel_battery_scale,
            "remote_panel_battery_voltage": remote_panel_battery_voltage,
            "remote_panel_battery_technology": remote_panel_battery_technology,
            "remote_panel_plugged": remote_panel_plugged,
            "remote_panel_battery_temperature": remote_panel_battery_temperature,
            "requestID": requestID,
            "responseTopic": responseTopic,
            "remoteMacAddess": remoteMacAddress,
        }

        response = await self.send_command(topic, payload, requestID)
        LOGGER.debug("MQTT: Receiving connect command")
        return response

    async def command_pingevent(self) -> None:
        LOGGER.debug("MQTT: Sending pingevent command")

        topic = "mastermeid"
        eventName = "pingevent"
        macAddress = self.settings.random_mac
        remote_panel_status = "Active"
        ipAddress = self.settings.plugin_ip
        current_battery_status = "Normal"
        remote_panel_battery_percentage = 100
        remote_panel_battery_temperature = 430
        remote_panel_battery_status = 3
        remote_panel_battery_scale = 100
        remote_panel_battery_voltage = 4102
        remote_panel_battery_present = True
        remote_panel_battery_technology = ""
        remote_panel_battery_level = 100
        remote_panel_battery_health = 2
        remote_panel_plugged = 1
        requestID = str(uuid.uuid4())
        remoteMacAddress = self.settings.random_mac
        responseTopic = "response_" + self.settings.random_mac

        payload = {
            "eventName": eventName,
            "macAddress": macAddress,
            "remote_panel_status": remote_panel_status,
            "ipAddress": ipAddress,
            "current_battery_status": current_battery_status,
            "remote_panel_battery_percentage": remote_panel_battery_percentage,
            "remote_panel_battery_temperature": remote_panel_battery_temperature,
            "remote_panel_battery_status": remote_panel_battery_status,
            "remote_panel_battery_scale": remote_panel_battery_scale,
            "remote_panel_battery_voltage": remote_panel_battery_voltage,
            "remote_panel_battery_present": remote_panel_battery_present,
            "remote_panel_battery_technology": remote_panel_battery_technology,
            "remote_panel_battery_level": remote_panel_battery_level,
            "remote_panel_battery_health": remote_panel_battery_health,
            "remote_panel_plugged": remote_panel_plugged,
            "requestID": requestID,
            "responseTopic": responseTopic,
            "remoteMacAddess": remoteMacAddress,
        }

        await self.send_command(topic, payload, requestID)
        LOGGER.debug("MQTT: Receiving pingevent command")

    async def command_timesync(self) -> None:
        LOGGER.debug("MQTT: Sending timeSync command")

        topic = "mastermeid"
        eventName = "timeSync"
        startTimestamp = datetime.datetime.now().timestamp()
        requestID = str(uuid.uuid4())
        responseTopic = "response_" + self.settings.random_mac
        remoteMacAddress = self.settings.random_mac

        payload = {
            "eventName": eventName,
            "startTimestamp": startTimestamp,
            "requestID": requestID,
            "responseTopic": responseTopic,
            "remoteMacAddess": remoteMacAddress,
        }

        await self.send_command(topic, payload, requestID)
        LOGGER.debug("MQTT: Receiving timeSync command")

    async def command_sync_database(self) -> dict:
        LOGGER.debug("MQTT: Sending syncdatabase command")

        topic = "mastermeid"
        eventName = "syncdatabase"
        requestID = str(uuid.uuid4())
        responseTopic = "response_" + self.settings.random_mac
        remoteMacAddress = self.settings.random_mac

        payload = {
            "eventName": eventName,
            "requestID": requestID,
            "responseTopic": responseTopic,
            "remoteMacAddess": remoteMacAddress,
        }

        response = await self.send_command(topic, payload, requestID)
        LOGGER.debug("MQTT: Receiving syncdatabase command")
        return response

    async def command_acstatus(self) -> None:
        LOGGER.debug("MQTT: Sending acStatus command")

        topic = "mastermeid"
        eventName = "acStatus"
        requestID = str(uuid.uuid4())
        responseTopic = "response_" + self.settings.random_mac
        remoteMacAddress = self.settings.random_mac
        acStatus = "Connected"

        payload = {"eventName": eventName,
                   "acStatus": acStatus,
                   "requestID": requestID,
                   "responseTopic": responseTopic,
                   "remoteMacAddess": remoteMacAddress}

        await self.send_command(topic, payload, requestID)

    async def command_dealer_logo(self) -> None:
        LOGGER.debug("MQTT: Sending dealerLogo command")

        topic = "mastermeid"
        eventName = "dealerLogo"
        requestID = str(uuid.uuid4())
        responseTopic = "response_" + self.settings.random_mac
        remoteMacAddress = self.settings.random_mac

        payload = {
            "eventName": eventName,
            "requestID": requestID,
            "responseTopic": responseTopic,
            "remoteMacAddess": remoteMacAddress,
        }

        await self.send_command(topic, payload, requestID)

    async def command_pair_status_request(self) -> None:
        LOGGER.debug("MQTT: Sending pair_status_request command")

        topic = "mastermeid"
        eventName = "pair_status_request"
        remoteMacAddress = self.settings.random_mac
        requestID = str(uuid.uuid4())
        responseTopic = "response_" + self.settings.random_mac

        payload = {
            "eventName": eventName,
            "requestID": requestID,
            "responseTopic": responseTopic,
            "remoteMacAddess": remoteMacAddress,
        }

        await self.send_command(topic, payload, requestID)
        LOGGER.debug("MQTT: Receiving pair_status_request command")

    async def command_disconnect(self) -> None:
        LOGGER.debug("MQTT: Sending disconnect command")

        topic = "mastermeid"
        eventName = "disconnect"
        remoteClientID = self.settings.mqtt_remote_client_id
        requestID = str(uuid.uuid4())
        remoteMacAddress = self.settings.random_mac

        payload = {
            "eventName": eventName,
            "remoteClientID": remoteClientID,
            "requestID": requestID,
            "remoteMacAddess": remoteMacAddress,
        }

        await self.send_command(topic, payload, requestID)

    async def command_pairing_request(self) -> dict:
        LOGGER.debug("MQTT: Sending pairing_request command")

        topic = "mastermeid"
        eventName = "connect_v204"
        pairing_request = True
        ipAddress = self.settings.plugin_ip
        macAddress = self.settings.random_mac
        remoteClientID = self.settings.mqtt_remote_client_id
        softwareVersion = "4.4.1"
        productType = "tab07_rk68"
        bssid = ""
        lastUpdateChecksum = "2132501716"
        dealerIconsCheckSum = ""
        remote_feature_support_version = "1"
        requestID = str(uuid.uuid4())
        responseTopic = "response_" + self.settings.random_mac
        remoteMacAddress = self.settings.random_mac

        dhcpInfo = {
            "ipaddress": "",
            "gateway": "",
            "netmask": "",
            "dns1": "",
            "dns2": "",
            "dhcpServer": "",
            "leaseDuration": "",
        }

        payload = {
            "eventName": eventName,
            "pairing_request": pairing_request,
            "ipAddress": ipAddress,
            "macAddress": macAddress,
            "remoteClientID": remoteClientID,
            "softwareVersion": softwareVersion,
            "producType": productType,
            "bssid": bssid,
            "dhcpInfo": json.dumps(dhcpInfo),
            "lastUpdateChecksum": lastUpdateChecksum,
            "dealerIconsCheckSum": dealerIconsCheckSum,
            "remote_feature_support_version": remote_feature_support_version,
            "requestID": requestID,
            "responseTopic": responseTopic,
            "remoteMacAddess": remoteMacAddress,
        }

        response = await self.send_command(topic, payload, requestID)
        LOGGER.debug("MQTT: Receiving pairing_request command")
        return response

    async def command_ui_delay(self, partition_id: str,silent_disarming:bool = False) -> None:
        LOGGER.debug("MQTT: Sending ui_delay command")

        # partition state needs to be sent for ui_delay to work
        partition = self.state.partition(partition_id)

        arming_command = {
            "operation_name": "ui_delay",
            "panel_status": partition.system_status,
            "userID": 0,
            "partitionID": partition_id,  # STR EXPECTED
            "silentDisarming":silent_disarming,
            "operation_source": 1,
            "macAddress": self.settings.random_mac,
        }

        topic = "mastermeid"
        eventName = "ipcCall"
        ipcServiceName = "qinternalservice"
        ipcInterfaceName = "android.os.IQInternalService"
        ipcTransactionID = 7
        requestID = str(uuid.uuid4())
        remoteMacAddress = self.settings.random_mac
        responseTopic = "response_" + self.settings.random_mac

        payload = {
            "eventName": eventName,
            "ipcServiceName": ipcServiceName,
            "ipcInterfaceName": ipcInterfaceName,
            "ipcTransactionID": ipcTransactionID,
            "ipcRequest": [{
                "dataType": "string",
                "dataValue":  json.dumps(arming_command),
            }],
            "requestID": requestID,
            "responseTopic": responseTopic,
            "remoteMacAddress": remoteMacAddress,
        }

        await self.send_command(topic, payload, requestID)
        LOGGER.debug("MQTT: Receiving ui_delay command")

    async def command_disarm(self, partition_id: str, user_code: str = "", silent_disarming: bool = False) -> bool:
        partition = self.state.partition(partition_id)
        if not partition:
            LOGGER.debug("MQTT: disarm command error - Unknow Partition")
            return False

        # Do local user code verification
        user_id = 1
        if self.check_user_code_on_disarm:
            user_id = self.panel.check_user(user_code)
            if user_id == -1:
                LOGGER.debug("MQTT: disarm command error - user_code error")
                return False

        async def get_mqtt_disarm_command(silent_disarming:bool) -> str:
            if partition.alarm_state == PartitionAlarmState.ALARM:
                return "disarm_from_emergency"
            if partition.system_status in {PartitionSystemStatus.ARM_AWAY_EXIT_DELAY,
                                           PartitionSystemStatus.ARM_STAY_EXIT_DELAY,
                                           PartitionSystemStatus.ARM_NIGHT_EXIT_DELAY}:
                return "disarm_from_openlearn_sensor"
            if partition.system_status in {PartitionSystemStatus.ARM_AWAY,
                                           PartitionSystemStatus.ARM_STAY,
                                           PartitionSystemStatus.ARM_NIGHT}:
                await self.command_ui_delay(partition_id,silent_disarming)
                return "disarm_the_panel_from_entry_delay"

            return "disarm_from_openlearn_sensor"

        mqtt_disarm_command = await get_mqtt_disarm_command(silent_disarming)
        LOGGER.debug("MQTT: Sending disarm command - check_user_code:%s", self.check_user_code_on_disarm)


        disarm_command = {
            "operation_name": mqtt_disarm_command,
            "userID": user_id,
            "partitionID": int(partition_id),  # INT EXPECTED
            "operation_source": 1,
            "macAddress": self.settings.random_mac,
        }

        topic = "mastermeid"
        eventName = "ipcCall"
        ipcServiceName = "qinternalservice"
        ipcInterfaceName = "android.os.IQInternalService"
        ipcTransactionID = 7
        requestID = str(uuid.uuid4())
        remoteMacAddress = self.settings.random_mac
        responseTopic = "response_" + self.settings.random_mac

        payload = {"eventName": eventName,
                   "ipcServiceName": ipcServiceName,
                   "ipcInterfaceName": ipcInterfaceName,
                   "ipcTransactionID": ipcTransactionID,
                   "ipcRequest": [{
                       "dataType": "string",
                       "dataValue":  json.dumps(disarm_command),
                   }],
                   "requestID": requestID,
                   "responseTopic": responseTopic,
                   "remoteMacAddress": remoteMacAddress}

        await self.send_command(topic, payload, requestID)
        LOGGER.debug("MQTT: Receiving disarm command")

        return True

    async def command_zwave_switch_multi_level(self, node_id: int, level: int) -> None:

        ipcRequest = [{
                "dataType": "int",  # Node ID
                "dataValue": node_id,
            },
            {
                "dataType": "int",  # ?
                "dataValue": 0,
            },
            {
                "dataType": "byteArray",  # ZWAVE MULTILEVELSWITCH COMMAND
                "dataValue": [38, 1, level],
            },
            {
                "dataType": "int",  # ?
                "dataValue": 0,
            },
            {
                "dataType": "int",  # ?
                "dataValue": 106,
            },
            {
                "dataType": "byteArray",
                "dataValue": [0],
            },
        ]

        topic = "mastermeid"
        eventName = "ipcCall"
        ipcServiceName = "qzwaveservice"
        ipcInterfaceName = "android.os.IQZwaveService"
        ipcTransactionID = 47
        requestID = str(uuid.uuid4())
        remoteMacAddress = self.settings.random_mac
        responseTopic = "response_" + self.settings.random_mac

        payload = {"eventName": eventName,
                   "ipcServiceName": ipcServiceName,
                   "ipcInterfaceName": ipcInterfaceName,
                   "ipcTransactionID": ipcTransactionID,
                   "ipcRequest": ipcRequest,
                   "requestID": requestID,
                   "responseTopic": responseTopic,
                   "remoteMacAddress": remoteMacAddress}

        await self.send_command(topic, payload, requestID)

    async def command_arm(self, partition_id: str, arming_type: str, user_code: str = "", exit_sounds: bool = False,
                          instant_arm: bool = False) -> bool:

        LOGGER.debug("MQTT: Sending arm command: partition%s, arming_type:%s, secure_arm:%s",
                     partition_id, arming_type, self.panel.SECURE_ARMING)

        user_id = 0

        partition = self.state.partition(partition_id)
        if not partition:
            LOGGER.debug("MQTT: arm command error - Unknow Partition")
            return False

        if self.panel.SECURE_ARMING == "true":
            # Do local user code verification to arm if secure arming is enabled
            user_id = self.panel.check_user(user_code)
            if user_id == -1:
                LOGGER.debug("MQTT: arm command error - user_code error")
                return False

        mqtt_arming_type = ""
        match arming_type:
            case "ARM-STAY":
                mqtt_arming_type = "ui_armstay"

            case "ARM-AWAY":
                mqtt_arming_type = "ui_armaway"

            case "ARM-NIGHT":
                mqtt_arming_type = "ui_armnight"

            case _:
                LOGGER.debug("MQTT: Sending arm command: Unknow arming_type:%s", arming_type)
                return False

        exitSoundValue = "ON"
        if not exit_sounds:
            exitSoundValue = "OFF"

        arming_command = {
            "operation_name": mqtt_arming_type,
            "bypass_zoneid_set": "[]",
            "userID": user_id,
            "partitionID": int(partition_id),
            "exitSoundValue":  exitSoundValue,
            "entryDelayValue": "OFF",
            "multiplePartitionsSelected": False,
            "instant_arming": instant_arm,
            "final_exit_arming_selected": False,
            "manually_selected_zones": "[]",
            "operation_source": 1,
            "macAddress": self.settings.random_mac,
        }

        topic = "mastermeid"
        eventName = "ipcCall"
        ipcServiceName = "qinternalservice"
        ipcInterfaceName = "android.os.IQInternalService"
        ipcTransactionID = 7
        requestID = str(uuid.uuid4())
        remoteMacAddress = self.settings.random_mac
        responseTopic = "response_" + self.settings.random_mac

        payload = {
            "eventName": eventName,
            "ipcServiceName": ipcServiceName,
            "ipcInterfaceName": ipcInterfaceName,
            "ipcTransactionID": ipcTransactionID,
            "ipcRequest": [{
                "dataType": "string",
                "dataValue":  json.dumps(arming_command),
            }],
            "requestID": requestID,
            "responseTopic": responseTopic,
            "remoteMacAddress": remoteMacAddress,
        }

        await self.send_command(topic, payload, requestID)

        return True
